#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pytest

import hyperopt

from polyaxon.polyflow.matrix.hyperopt import V1Hyperopt
from polyaxon.polytune.search_managers.hyperopt.manager import HyperoptManager
from tests.utils import BaseTestCase


@pytest.mark.polytune_mark
class TestHyperoptSearch(BaseTestCase):
    def test_hyperopt_search_config(self):
        assert HyperoptManager.CONFIG == V1Hyperopt

    def test_hyperopt_algorithm(self):
        config = V1Hyperopt.from_dict(
            {
                "concurrency": 2,
                "algorithm": "rand",
                "numRuns": 1,
                "params": {"param": {"kind": "uniform", "value": [0.01, 0.5]}},
            }
        )
        manager = HyperoptManager(config)
        assert manager.algorithm == hyperopt.rand.suggest

        config = V1Hyperopt.from_dict(
            {
                "concurrency": 2,
                "algorithm": "anneal",
                "numRuns": 1,
                "params": {"param": {"kind": "uniform", "value": [0.01, 0.5]}},
            }
        )
        manager = HyperoptManager(config)
        assert manager.algorithm == hyperopt.anneal.suggest

        config = V1Hyperopt.from_dict(
            {
                "concurrency": 2,
                "algorithm": "tpe",
                "numRuns": 1,
                "params": {"param": {"kind": "uniform", "value": [0.01, 0.5]}},
            }
        )
        manager = HyperoptManager(config)
        assert manager.algorithm == hyperopt.tpe.suggest

    def test_search_space(self):
        config = V1Hyperopt.from_dict(
            {
                "concurrency": 2,
                "algorithm": "rand",
                "numRuns": 1,
                "params": {
                    "param1": {"kind": "uniform", "value": [0.01, 0.5]},
                    "param2": {"kind": "quniform", "value": [0.01, 0.99, 0.1]},
                    "param3": {"kind": "normal", "value": [0, 0.99]},
                    "param4": {"kind": "choice", "value": [32, 64, 126, 256]},
                    "param5": {
                        "kind": "choice",
                        "value": ["sgd", "adagrad", "adam", "ftrl"],
                    },
                    "param6": {"kind": "linspace", "value": [0, 10, 1]},
                    "param7": {"kind": "geomspace", "value": [0.1, 1, 1]},
                },
            }
        )
        manager = HyperoptManager(config)
        assert set(manager._param_to_value.keys()) == {
            "param4",
            "param5",
            "param6",
            "param7",
        }
        assert set(manager._search_space.keys()) == {
            "param1",
            "param2",
            "param3",
            "param4",
            "param5",
            "param6",
            "param7",
        }

    def test_get_rand_suggestions(self):
        config = V1Hyperopt.from_dict(
            {
                "concurrency": 2,
                "algorithm": "rand",
                "numRuns": 1,
                "params": {
                    "lr": {"kind": "uniform", "value": [0.01, 0.5]},
                    "dropout": {"kind": "uniform", "value": [0.01, 0.99]},
                    "batch": {"kind": "choice", "value": [32, 64, 126, 256]},
                    "optimizer": {
                        "kind": "choice",
                        "value": ["sgd", "adagrad", "adam", "ftrl"],
                    },
                },
            }
        )

        suggestion = HyperoptManager(config).get_suggestions()[0]

        self.assertTrue(0.99 >= suggestion["dropout"] >= 0.01)
        self.assertTrue(0.5 >= suggestion["lr"] >= 0.01)
        self.assertTrue(suggestion["batch"] in [32, 64, 126, 256])
        self.assertTrue(suggestion["optimizer"] in ["sgd", "adagrad", "adam", "ftrl"])

        config = V1Hyperopt.from_dict(
            {
                "concurrency": 2,
                "algorithm": "rand",
                "numRuns": 10,
                "params": {
                    "lr": {"kind": "uniform", "value": [0.01, 0.5]},
                    "dropout": {"kind": "uniform", "value": [0.01, 0.99]},
                    "batch": {"kind": "choice", "value": [32, 64, 126, 256]},
                    "optimizer": {
                        "kind": "choice",
                        "value": ["sgd", "adagrad", "adam", "ftrl"],
                    },
                },
            }
        )

        assert len(HyperoptManager(config).get_suggestions()) == 10

    def test_get_anneal_suggestions(self):
        config = V1Hyperopt.from_dict(
            {
                "concurrency": 2,
                "algorithm": "anneal",
                "numRuns": 1,  # TODO: no numRuns
                "params": {
                    "lr": {"kind": "uniform", "value": [0.01, 0.5]},
                    "dropout": {"kind": "uniform", "value": [0.01, 0.99]},
                    "batch": {"kind": "choice", "value": [32, 64, 126, 256]},
                    "optimizer": {
                        "kind": "choice",
                        "value": ["sgd", "adagrad", "adam", "ftrl"],
                    },
                },
            }
        )

        suggestion = HyperoptManager(config).get_suggestions()[0]

        self.assertTrue(0.99 >= suggestion["dropout"] >= 0.01)
        self.assertTrue(0.5 >= suggestion["lr"] >= 0.01)
        self.assertTrue(suggestion["batch"] in [32, 64, 126, 256])
        self.assertTrue(suggestion["optimizer"] in ["sgd", "adagrad", "adam", "ftrl"])

    def test_get_tpe_suggestions(self):
        config = V1Hyperopt.from_dict(
            {
                "concurrency": 2,
                "algorithm": "tpe",
                "numRuns": 1,
                "params": {
                    "lr": {"kind": "uniform", "value": [0.01, 0.5]},
                    "dropout": {"kind": "uniform", "value": [0.01, 0.99]},
                    "batch": {"kind": "choice", "value": [32, 64, 126, 256]},
                    "optimizer": {
                        "kind": "choice",
                        "value": ["sgd", "adagrad", "adam", "ftrl"],
                    },
                },
            }
        )

        suggestion = HyperoptManager(config).get_suggestions()[0]

        self.assertTrue(0.99 >= suggestion["dropout"] >= 0.01)
        self.assertTrue(0.5 >= suggestion["lr"] >= 0.01)
        self.assertTrue(suggestion["batch"] in [32, 64, 126, 256])
        self.assertTrue(suggestion["optimizer"] in ["sgd", "adagrad", "adam", "ftrl"])

        config = V1Hyperopt.from_dict(
            {
                "concurrency": 2,
                "algorithm": "tpe",
                "numRuns": 10,
                "params": {
                    "lr": {"kind": "uniform", "value": [0.01, 0.5]},
                    "dropout": {"kind": "uniform", "value": [0.01, 0.99]},
                    "batch": {"kind": "choice", "value": [32, 64, 126, 256]},
                    "optimizer": {
                        "kind": "choice",
                        "value": ["sgd", "adagrad", "adam", "ftrl"],
                    },
                },
            }
        )

        assert len(HyperoptManager(config).get_suggestions()) == 10



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pytest

from marshmallow.exceptions import ValidationError

from polyaxon.polyflow.matrix import V1Hyperband
from polyaxon.polyflow.optimization import V1Optimization, V1OptimizationMetric
from tests.utils import BaseTestCase, assert_equal_dict


@pytest.mark.workflow_mark
class TestWorkflowV1Hyperbands(BaseTestCase):
    def test_hyperband_config(self):
        config_dict = {
            "kind": "hyperband",
            "maxIterations": 10,
            "eta": 3,
            "resource": {"name": "steps", "type": "int"},
            "resume": False,
            "metric": V1OptimizationMetric(
                name="loss", optimization=V1Optimization.MINIMIZE
            ).to_dict(),
            "params": {"lr": {"kind": "choice", "value": [[0.1], [0.9]]}},
        }
        config = V1Hyperband.from_dict(config_dict)
        assert_equal_dict(config.to_dict(), config_dict)

        # Raises for negative values
        config_dict["maxIterations"] = 0
        with self.assertRaises(ValidationError):
            V1Hyperband.from_dict(config_dict)

        config_dict["maxIterations"] = -0.5
        with self.assertRaises(ValidationError):
            V1Hyperband.from_dict(config_dict)

        config_dict["maxIterations"] = 3
        # Add numRuns percent
        config_dict["eta"] = -0.5
        with self.assertRaises(ValidationError):
            V1Hyperband.from_dict(config_dict)

        config_dict["eta"] = 2.9
        config = V1Hyperband.from_dict(config_dict)
        assert_equal_dict(config.to_dict(), config_dict)



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from typing import Optional

import polyaxon_sdk

from marshmallow import fields, pre_load, validate

from polyaxon.connections.kinds import V1ConnectionKind
from polyaxon.connections.schemas import (
    ConnectionSchema,
    K8sResourceSchema,
    V1BucketConnection,
    V1ClaimConnection,
    V1GitConnection,
    V1HostConnection,
    V1HostPathConnection,
)
from polyaxon.schemas.base import BaseCamelSchema, BaseConfig
from polyaxon.schemas.types.k8s_resources import V1K8sResourceType


class ConnectionTypeSchema(BaseCamelSchema):
    name = fields.Str(required=True)
    kind = fields.Str(
        required=True, validate=validate.OneOf(V1ConnectionKind.allowable_values)
    )
    description = fields.Str(allow_none=True)
    tags = fields.List(fields.Str(), allow_none=True)
    schema = fields.Nested(ConnectionSchema, allow_none=True)
    secret = fields.Nested(K8sResourceSchema, allow_none=True)
    config_map = fields.Nested(K8sResourceSchema, allow_none=True)

    @staticmethod
    def schema_config():
        return V1ConnectionType

    @pre_load
    def pre_make(self, data, **kwargs):
        schema = data.get("schema")
        kind = data.get("kind")
        if schema and kind:
            schema["kind"] = "custom"
            if kind in V1ConnectionKind.BLOB_VALUES:
                schema["kind"] = V1BucketConnection.IDENTIFIER

            if kind == V1ConnectionKind.VOLUME_CLAIM:
                schema["kind"] = V1ClaimConnection.IDENTIFIER

            if kind == V1ConnectionKind.HOST_PATH:
                schema["kind"] = V1HostPathConnection.IDENTIFIER

            if kind == V1ConnectionKind.REGISTRY:
                schema["kind"] = V1HostConnection.IDENTIFIER

            if kind == V1ConnectionKind.GIT:
                schema["kind"] = V1GitConnection.IDENTIFIER

        return data


class V1ConnectionType(BaseConfig, polyaxon_sdk.V1ConnectionType):
    """Connections are how Polyaxon connects several
    types of external systems and resources to your operations.

    All connections in Polyaxon are typed, and some of them have special built-in handlers
    to automatically connect and load information.

    Using connections you can define the boilerplate required
    to connect a volume, a blob storage once, secret definition for loading data from a data source.
    Rnd users, e.g. data scientist, can just reference the name of the connection to use it
    without dealing with the configuration every time.

    Connections are not required to mount secrets or configurations,
    in fact users can also mount secrets and volumes the Kubernetes way,
    but this mean you are exposing the service to very few team members who have
    the Kubernetes know-how, in some advance use-cases,
    you will have to leverage the low level Kubernetes API,
    but for most interactions, using the connection specification is much simpler,
    similar to how easy it is to define service accounts or image pull secrets instead of
    defining all the volumes and mounting them to the containers manually.

    For some distributions, Polyaxon will exposes:
        * Analytics about how often connections are used.
        * Jobs that requested those connections.
        * Profiling and run time meta data to optimize access to those resources and connections.
        * Additional RBAC and ACL rules to control who can access the connections.

    Args:
         name: str
         description: str, optional
         tags: List[str], optional
         kind: str, Union[`host_path`, `volume_claim`, `gcs`, `s3`, `wasb`, `registry`, `git`,
                          `aws`, `gcp`, `azure`, `mysql`, `postgres`, `oracle`, `vertica`,
                          `sqlite`, `mssql`, `redis`, `presto`, `mongo`, `cassandra`, `ftp`,
                          `grpc`, `hdfs`, `http`, `pig_cli`, `hive_cli`, `hive_metastore`,
                          `hive_server2`, `jdbc`, `jenkins`, `samba`, `snowflake`, `ssh`,
                          `cloudant`, `databricks`, `segment`, `slack`, `discord`, `mattermost`,
                          `pagerduty`, `hipchat`, `webhook`, `custom`]
        schema: dict, optional
        secret: str, optional
        config_map: str, optional


    ## YAML usage

    ```yaml
    >>> artifactsStore:
    >>>   name: azure
    >>>   kind: wasb
    >>>   schema:
    >>>     bucket: "wasbs://test@container.blob.core.windows.net/"
    >>>   secret:
    >>>     name: "az-secret"
    >>> connections:
    >>>   - name: repo-test
    >>>     description: "some description"
    >>>     tags: ["tag1", "tag2"]
    >>>     kind: git
    >>>     schema:
    >>>       url: https://gitlab.com/org/test
    >>>     secret:
    >>>       name: "gitlab-connection"
    >>>   - name: docker-connection
    >>>     description: "some description"
    >>>     kind: registry
    >>>     schema:
    >>>       url: org/repo
    >>>     secret:
    >>>       name: docker-conf
    >>>       mountPath: /kaniko/.docker
    >>> notificationConnections:
    >>>   - name: my-slack
    >>>     kind: slack
    >>>     secret:
    >>>       name: my-slack
    ```

    ## Fields

    ### name

    The connection name must be unique within an organization.
    User can use this unique name to reference a connection in their components and operations.

    An end user does not need to know about how to mount secrets and configurations
    to access a dataset for example, they can just reference the name of the connection.

    ### description

    A short description about the purpose of the connection for other users.

    For example, "s3 bucket with radio images"

    ### tags

    Tags to categorize the connection in the connections catalog table.

    ### kind

    the kind of the connection. Apart from the fact that Polyaxon
    has built-in handlers for several connections, user can build their own handlers,
    for example you can create a handler for pulling data from a database
    or a data lake based on a specific kind.

    Polyaxon will show a small connection logo for some types in the
    dashboard and analytics about the connection usage.

    Polyaxon exposes this list of connection kinds:
    [`host_path`, `volume_claim`, `gcs`, `s3`, `wasb`, `registry`, `git`, `aws`,
     `gcp`, `azure`, `mysql`, `postgres`, `oracle`, `vertica`,
     `sqlite`, `mssql`, `redis`, `presto`, `mongo`, `cassandra`, `ftp`,
     `grpc`, `hdfs`, `http`, `pig_cli`, `hive_cli`, `hive_metastore`,
     `hive_server2`, `jdbc`, `jenkins`, `samba`, `snowflake`, `ssh`,
     `cloudant`, `databricks`, `segment`, `slack`, `discord`, `mattermost`,
     `pagerduty`, `hipchat`, `webhook`, `custom`]

    Polyaxon can also automatically handle these connection kinds:
    [`host_path`, `volume_claim`, `gcs`, `s3`, `wasb`, `registry`, `git`]

    ### schema

    In order to leverage some built-in functionalities in Polyaxon,
    e.g. automatic management of outputs, initializers for cloning code from git repos,
    loading data from S3/GCS/Azure/Volumes/Paths, or pushing container images to a registry,
    the schema is how Polyaxon knows how to authenticate the containers that will handle that logic.

    If you opt-out of using those functionalities, you can leave this field empty or
    you can expose any key/value object you want for your own custom handlers.

    For more details please check connection schema section for the built-in handlers:
        * [artifacts connections](/docs/setup/connections/artifacts/)
        * [git connections](/docs/setup/connections/git/)
        * [docker registry connections](/docs/setup/connections/registry/)

    ### secret

    We assume that each connection will only need to access to at most one secret.
    If you are building a specific handler for a connection,
    this is where you will need to expose the necessary paths or environment variables needed
    to make the http/grpc connection.

    In many cases you might not need to expose any secret, for instance for volumes and host paths.

    The connection secret schema has 3 fields:

        * name: str, required, the name of the secret,
                this is the minimum to tell Polyaxon to mount that secret
                whenever the connection is referenced.
        * mountPath: str, optional, if you prefer to mount the secret as a volume
                     instead of exposing its items as environment variables.
        * items: List[str], optional, if you only want to expose a subset
                 of the items in the secret.

    Example slack connection

    ```yaml
    >>> name: my-slack
    >>> kind: slack
    >>> secret:
    >>>   name: my-slack
    ```

    Example docker connection with mountPath

    ```yaml
    >>> kind: registry
    >>> schema:
    >>>   url: registry.com/org/repo
    >>> secret:
    >>>   name: docker-conf
    >>>   mountPath: /kaniko/.docker
    ```

    ### configMap

    We assume that each connection will only need to access to at most one config map.
    Similar logic for the secret, if you need to expose more information to connect to a service,
    you can reference a config map.

    In many cases you might not need to expose any config map.

    The connection configMap schema has 3 fields:

        * name: str, required, the name of the configMap,
                this is the minimum to tell Polyaxon to mount that configMap
                whenever the connection is referenced.
        * mountPath: str, optional, if you prefer to mount the configMap as a volume
                     instead of exposing its items as environment variables.
        * items: List[str], optional, if you only want to expose a subset
                 of the items in the configMap.
    """

    IDENTIFIER = "connection"
    SCHEMA = ConnectionTypeSchema
    REDUCED_ATTRIBUTES = [
        "name",
        "kind",
        "description",
        "tags",
        "schema",
        "secret",
        "configMap",
    ]

    @classmethod
    def from_model(cls, model) -> "V1ConnectionType":
        schema = model.schema
        secret = model.secret
        config_map = model.config_map
        if hasattr(schema, "to_dict"):
            schema = schema.to_dict()
        if hasattr(secret, "to_dict"):
            secret = secret.to_dict()
        if hasattr(config_map, "to_dict"):
            config_map = config_map.to_dict()
        return V1ConnectionType.from_dict(
            {
                "name": model.name,
                "kind": model.kind,
                "schema": schema,
                "secret": secret,
                "configMap": config_map,
            }
        )

    @property
    def store_path(self) -> str:
        if self.is_mount:
            return self.schema.mount_path
        if self.is_bucket:
            return self.schema.bucket

    @property
    def is_mount(self) -> bool:
        return V1ConnectionKind.is_mount(self.kind)

    @property
    def is_artifact(self) -> bool:
        return V1ConnectionKind.is_artifact(self.kind)

    @property
    def is_host_path(self) -> bool:
        return V1ConnectionKind.is_host_path(self.kind)

    @property
    def is_volume_claim(self) -> bool:
        return V1ConnectionKind.is_volume_claim(self.kind)

    @property
    def is_bucket(self) -> bool:
        return V1ConnectionKind.is_bucket(self.kind)

    @property
    def is_gcs(self) -> bool:
        return self.kind == V1ConnectionKind.GCS

    @property
    def is_s3(self) -> bool:
        return self.kind == V1ConnectionKind.S3

    @property
    def is_wasb(self) -> bool:
        return V1ConnectionKind.is_wasb(self.kind)

    def get_secret(self) -> Optional[V1K8sResourceType]:
        if self.secret:
            return V1K8sResourceType(name=self.secret.name, schema=self.secret)
        return None

    def get_config_map(self) -> Optional[V1K8sResourceType]:
        if self.config_map:
            return V1K8sResourceType(name=self.config_map.name, schema=self.config_map)
        return None



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from marshmallow import ValidationError, fields


class Tensor(fields.Field):
    def _deserialize(self, value, attr, data, **kwargs):
        if isinstance(value, str):
            return [value, 0, 0]
        if isinstance(value, list) and len(value) == 3:
            condition = (
                isinstance(value[0], str)
                and isinstance(value[1], int)
                and isinstance(value[1], int)
            )
            if condition:
                return value
        raise ValidationError("This field expects a str or a list of [str, int, int].")



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Dict

from polyaxon.agents.spawners.base import BaseSpawner
from polyaxon.k8s.custom_resources.operation import get_resource_name
from polyaxon.k8s.manager import K8SManager


class Spawner(BaseSpawner):
    @property
    def k8s_manager(self):
        if not self._k8s_manager:
            self._k8s_manager = K8SManager(
                k8s_config=self.k8s_config,
                namespace=self.namespace,
                in_cluster=self.in_cluster,
            )
        return self._k8s_manager

    def create(self, run_uuid: str, run_kind: str, resource: Dict) -> Dict:
        mixin = self._get_mixin_for_kind(kind=run_kind)
        resource_name = get_resource_name(run_uuid)
        return self.k8s_manager.create_custom_object(
            name=resource_name,
            group=mixin.GROUP,
            version=mixin.API_VERSION,
            plural=mixin.PLURAL,
            body=resource,
        )

    def apply(self, run_uuid: str, run_kind: str, resource: Dict) -> Dict:
        mixin = self._get_mixin_for_kind(kind=run_kind)
        resource_name = get_resource_name(run_uuid)
        return self.k8s_manager.update_custom_object(
            name=resource_name,
            group=mixin.GROUP,
            version=mixin.API_VERSION,
            plural=mixin.PLURAL,
            body=resource,
        )

    def stop(self, run_uuid: str, run_kind: str):
        mixin = self._get_mixin_for_kind(kind=run_kind)
        resource_name = get_resource_name(run_uuid)
        self.k8s_manager.delete_custom_object(
            name=resource_name,
            group=mixin.GROUP,
            version=mixin.API_VERSION,
            plural=mixin.PLURAL,
        )

    def clean(self, run_uuid: str, run_kind: str):
        return self.apply(
            run_uuid=run_uuid,
            run_kind=run_kind,
            resource={"metadata": {"finalizers": None}},
        )

    def get(self, run_uuid: str, run_kind: str):
        mixin = self._get_mixin_for_kind(kind=run_kind)
        resource_name = get_resource_name(run_uuid)
        self.k8s_manager.get_custom_object(
            name=resource_name,
            group=mixin.GROUP,
            version=mixin.API_VERSION,
            plural=mixin.PLURAL,
        )



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from typing import Dict, Union

import polyaxon_sdk

from polyaxon import settings
from polyaxon.client import PolyaxonClient
from polyaxon.client.decorators import check_no_op, check_offline
from polyaxon.constants import DEFAULT
from polyaxon.env_vars.getters import get_project_full_name, get_project_info
from polyaxon.exceptions import PolyaxonClientException
from polyaxon.utils.query_params import get_query_params


class ProjectClient:
    """ProjectClient is a client to communicate with Polyaxon projects endpoints.

    If no values are passed to this class,
    Polyaxon will try to resolve the owner and project from the environment:
        * If you have a configured CLI, Polyaxon will use the configuration of the cli.
        * If you have a cached project using the CLI,
        the client will default to that cached project unless you override the values.
        * If you use this client in the context of a job or a service managed by Polyaxon,
        a configuration will be available to resolve the values based on that run.

    If you intend to create a new project instance or to list projects,
    only the `owner` parameter is required.

    Properties:
        project: str.
        owner: str.
        project_data: V1Project.

    Args:
        owner: str, optional, the owner is the username or
               the organization name owning this project.
        project: str, optional, project name.
        client: [PolyaxonClient](/docs/core/python-library/polyaxon-client/), optional,
                an instance of a configured client, if not passed,
                a new instance will be created based on the available environment.

    Raises:
        PolyaxonClientException: If no owner is passed and Polyaxon cannot
            resolve an owner from the environment.
    """

    @check_no_op
    def __init__(
        self,
        owner: str = None,
        project: str = None,
        client: PolyaxonClient = None,
    ):
        if not owner and project:
            owner, project = get_project_info(
                get_project_full_name(owner=owner, project=project)
            )

        if not owner:
            raise PolyaxonClientException("Please provide a valid project owner.")

        self.client = client
        if not (self.client or settings.CLIENT_CONFIG.is_offline):
            self.client = PolyaxonClient()

        self._owner = owner or DEFAULT
        self._project = project
        self._project_data = polyaxon_sdk.V1Project()

    @property
    def owner(self):
        return self._owner

    @property
    def project(self):
        return self._project

    @property
    def project_data(self):
        return self._project_data

    @check_no_op
    @check_offline
    def refresh_data(self):
        """Fetches the project data from the api."""
        self._project_data = self.client.projects_v1.get_project(
            self.owner, self.project
        )
        if self._project_data.owner is None:
            self._project_data.owner = self.owner

    @check_no_op
    @check_offline
    def create(self, data: Union[Dict, polyaxon_sdk.V1Project]):
        """Creates a new project based on the data passed.

        [Project API](/docs/api/#operation/CreateProject)

        Args:
            data: dict or V1Project, required.

        Returns:
            V1Project, project instance from the response.
        """
        self._project_data = self.client.projects_v1.create_project(self.owner, data)
        self._project_data.owner = self.owner
        self._project = self._project_data.name
        return self._project_data

    @check_no_op
    @check_offline
    def list(
        self, query: str = None, sort: str = None, limit: int = None, offset: int = None
    ):
        """Lists projects under the current owner.

        [Project API](/docs/api/#operation/ListProjects)

        Args:
            query: str, optional, query filters, please refer to
                    [Project PQL](/docs/core/query-language/projects/#query)
            sort: str, optional, fields to order by, please refer to
                    [Project PQL](/docs/core/query-language/projects/#sort)
            limit: int, optional, limit of projects to return.
            offset: int, optional, offset pages to paginate projects.

        Returns:
            List[V1Project], list of project instances.
        """
        params = get_query_params(limit=limit, offset=offset, query=query, sort=sort)
        return self.client.projects_v1.list_projects(self.owner, **params)

    @check_no_op
    @check_offline
    def delete(self):
        """Deletes project based on the current owner and project."""
        return self.client.projects_v1.delete_project(self.owner, self.project)

    @check_no_op
    @check_offline
    def update(self, data: Union[Dict, polyaxon_sdk.V1Project]):
        """Updates a project based on the data passed.

        [Project API](/docs/api/#operation/PatchProject)

        Args:
            data: Dict or V1Project, required.

        Returns:
            V1Project, project instance from the response.
        """
        self._project_data = self.client.projects_v1.patch_project(
            self.owner, self.project, body=data
        )
        self._project = self._project_data.name
        return self._project_data



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from marshmallow.exceptions import ValidationError

from polyaxon.connections.kinds import V1ConnectionKind
from polyaxon.connections.schemas import (
    V1BucketConnection,
    V1ClaimConnection,
    V1GitConnection,
    V1HostConnection,
    V1HostPathConnection,
    validate_connection,
)
from tests.utils import BaseTestCase


class TestV1BucketConnection(BaseTestCase):
    def test_claim_connect_config(self):
        config_dict = {}
        with self.assertRaises(ValidationError):
            V1BucketConnection.from_dict(config_dict)

        with self.assertRaises(ValidationError):
            validate_connection(V1ConnectionKind.VOLUME_CLAIM, config_dict)

        config_dict = {"bucket": "sdf"}
        config = V1BucketConnection.from_dict(config_dict)
        assert config.to_dict() == config_dict

        with self.assertRaises(ValidationError):
            validate_connection(V1ConnectionKind.VOLUME_CLAIM, config_dict)

        validate_connection(V1ConnectionKind.S3, config_dict)
        validate_connection(V1ConnectionKind.GCP, config_dict)
        validate_connection(V1ConnectionKind.WASB, config_dict)


class TestV1ClaimConnection(BaseTestCase):
    def test_claim_connect_config(self):
        config_dict = {}
        with self.assertRaises(ValidationError):
            V1ClaimConnection.from_dict(config_dict)

        with self.assertRaises(ValidationError):
            validate_connection(V1ConnectionKind.VOLUME_CLAIM, config_dict)

        config_dict = {"volumeClaim": "foo"}
        with self.assertRaises(ValidationError):
            V1ClaimConnection.from_dict(config_dict)

        with self.assertRaises(ValidationError):
            validate_connection(V1ConnectionKind.VOLUME_CLAIM, config_dict)

        config_dict = {"volumeClaim": "foo", "mountPath": "foo", "readOnly": True}
        config = V1ClaimConnection.from_dict(config_dict)
        assert config.to_dict() == config_dict

        with self.assertRaises(ValidationError):
            validate_connection(V1ConnectionKind.S3, config_dict)

        with self.assertRaises(ValidationError):
            validate_connection(V1ConnectionKind.WASB, config_dict)

        validate_connection(V1ConnectionKind.VOLUME_CLAIM, config_dict)


class TestV1HostPathConnection(BaseTestCase):
    def test_host_path_connect_config(self):
        config_dict = {}
        with self.assertRaises(ValidationError):
            V1HostPathConnection.from_dict(config_dict)

        with self.assertRaises(ValidationError):
            validate_connection(V1ConnectionKind.HOST_PATH, config_dict)

        config_dict = {"host_path": "foo"}
        with self.assertRaises(ValidationError):
            V1HostPathConnection.from_dict(config_dict)

        with self.assertRaises(ValidationError):
            validate_connection(V1ConnectionKind.HOST_PATH, config_dict)

        config_dict = {"hostPath": "foo", "mountPath": "foo", "readOnly": True}
        config = V1HostPathConnection.from_dict(config_dict)
        assert config.to_dict() == config_dict

        with self.assertRaises(ValidationError):
            validate_connection(V1ConnectionKind.S3, config_dict)

        with self.assertRaises(ValidationError):
            validate_connection(V1ConnectionKind.WASB, config_dict)

        validate_connection(V1ConnectionKind.HOST_PATH, config_dict)


class TestV1HostConnection(BaseTestCase):
    def test_host_connect_config(self):
        config_dict = {}
        with self.assertRaises(ValidationError):
            V1HostConnection.from_dict(config_dict)

        with self.assertRaises(ValidationError):
            validate_connection(V1ConnectionKind.REGISTRY, config_dict)

        config_dict = {"url": "foo", "insecure": True}
        config = V1HostConnection.from_dict(config_dict)
        assert config.to_dict() == config_dict

        with self.assertRaises(ValidationError):
            validate_connection(V1ConnectionKind.GIT, config_dict)

        with self.assertRaises(ValidationError):
            validate_connection(V1ConnectionKind.S3, config_dict)

        with self.assertRaises(ValidationError):
            validate_connection(V1ConnectionKind.WASB, config_dict)

        validate_connection(V1ConnectionKind.REGISTRY, config_dict)


class TestV1GitConnection(BaseTestCase):
    def test_git_connect_config(self):
        config_dict = {}
        V1GitConnection.from_dict(config_dict)
        validate_connection(V1ConnectionKind.GIT, config_dict)

        config_dict = {"url": "foo"}
        config = V1GitConnection.from_dict(config_dict)
        assert config.to_dict() == config_dict

        config_dict = {"url": "foo", "revision": "foo"}
        config = V1GitConnection.from_dict(config_dict)
        assert config.to_dict() == config_dict

        with self.assertRaises(ValidationError):
            validate_connection(V1ConnectionKind.REGISTRY, config_dict)

        with self.assertRaises(ValidationError):
            validate_connection(V1ConnectionKind.S3, config_dict)

        with self.assertRaises(ValidationError):
            validate_connection(V1ConnectionKind.WASB, config_dict)

        validate_connection(V1ConnectionKind.GIT, config_dict)



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from collections.abc import Mapping

from polyaxon.config_reader.spec import ConfigSpec
from polyaxon.polyaxonfile.specs import kinds as spec_kinds
from polyaxon.polyaxonfile.specs.base import BaseSpecification
from polyaxon.polyaxonfile.specs.compiled_operation import (
    CompiledOperationSpecification,
)
from polyaxon.polyaxonfile.specs.component import ComponentSpecification
from polyaxon.polyaxonfile.specs.operation import OperationSpecification

SPECIFICATION_BY_KIND = {
    spec_kinds.OPERATION: OperationSpecification,
    spec_kinds.COMPILED_OPERATION: CompiledOperationSpecification,
    spec_kinds.COMPONENT: ComponentSpecification,
}


def get_specification(data):
    if not isinstance(data, Mapping):
        data = ConfigSpec.read_from(data)
    kind = BaseSpecification.get_kind(data=data)
    return SPECIFICATION_BY_KIND[kind].read(data)



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
import logging
import time

from docker import APIClient
from docker.errors import APIError, BuildError
from urllib3.exceptions import ReadTimeoutError

from polyaxon.exceptions import PolyaxonBuildException
from polyaxon.schemas.types import V1UriType
from polyaxon.utils.log_levels import LogLevels

_logger = logging.getLogger("polyaxon.dockerizer")


class DockerMixin:
    IS_BUILD = None

    def _prepare_log_lines(self, log_line):  # pylint:disable=too-many-branches
        raw = log_line.decode("utf-8").strip()
        raw_lines = raw.split("\n")
        log_lines = []
        status = True
        for raw_line in raw_lines:
            try:
                json_line = json.loads(raw_line)

                if json_line.get("error"):
                    log_lines.append(
                        "{}: {}".format(
                            LogLevels.ERROR, str(json_line.get("error", json_line))
                        )
                    )
                    status = False
                else:
                    if json_line.get("stream"):
                        log_lines.append(
                            "Building: {}".format(json_line["stream"].strip())
                        )
                    elif json_line.get("status"):
                        if not self.is_pushing and not self.IS_BUILD:
                            self.is_pushing = True
                            log_lines.append("Pushing ...")
                    elif json_line.get("aux"):
                        log_lines.append(
                            "Pushing finished: {}".format(json_line.get("aux"))
                        )
                    else:
                        log_lines.append(str(json_line))
            except json.JSONDecodeError:
                log_lines.append("JSON decode error: {}".format(raw_line))
        return log_lines, status

    def _handle_logs(self, log_lines):
        for log_line in log_lines:
            print(log_line)  # pylint:disable=superfluous-parens

    def _handle_log_stream(self, stream):
        log_lines = []
        status = True
        try:
            for log_line in stream:
                new_log_lines, new_status = self._prepare_log_lines(log_line)
                log_lines += new_log_lines
                if not new_status:
                    status = new_status
                self._handle_logs(log_lines)
                log_lines = []
            if log_lines:
                self._handle_logs(log_lines)
        except (BuildError, APIError) as e:
            self._handle_logs(
                [
                    "{}: Could not build the image, encountered {}".format(
                        LogLevels.ERROR, e
                    )
                ]
            )
            return False

        return status


class DockerBuilder(DockerMixin):
    IS_BUILD = True

    def __init__(
        self, context, destination, credstore_env=None, registries=None, docker=None
    ):
        self.destination = destination

        self.context = context
        self._validate_registries(registries)
        self.registries = registries
        self.docker = docker or APIClient(version="auto", credstore_env=credstore_env)
        self.is_pushing = False

    @staticmethod
    def _validate_registries(registries):
        if not registries or isinstance(registries, V1UriType):
            return True

        for registry in registries:
            if not isinstance(registry, V1UriType):
                raise PolyaxonBuildException(
                    "A registry `{}` is not valid Urispec.".format(registry)
                )

        return True

    def check_image(self):
        return self.docker.images(self.destination)

    def login_private_registries(self):
        if not self.registries:
            return
        for registry in self.registries:
            self.docker.login(
                username=registry.user,
                password=registry.password,
                registry=registry.host,
                reauth=True,
            )

    def build(self, nocache=False, memory_limit=None):
        limits = {
            # Disable memory swap for building
            "memswap": -1
        }
        if memory_limit:
            limits["memory"] = memory_limit

        stream = self.docker.build(
            path=self.context,
            tag=self.destination,
            forcerm=True,
            rm=True,
            pull=True,
            nocache=nocache,
            container_limits=limits,
        )
        return self._handle_log_stream(stream=stream)


class DockerPusher(DockerMixin):
    IS_BUILD = False

    def __init__(self, destination, credstore_env=None, docker=None):
        self.destination = destination
        self.docker = docker or APIClient(version="auto", credstore_env=credstore_env)
        self.is_pushing = False

    def push(self):
        stream = self.docker.push(self.destination, stream=True)
        return self._handle_log_stream(stream=stream)


def _build(
    context, destination, nocache, docker=None, credstore_env=None, registries=None
):
    """Build necessary code for a job to run"""
    _logger.info("Starting build ...")

    # Build the image
    docker_builder = DockerBuilder(
        context=context,
        destination=destination,
        credstore_env=credstore_env,
        registries=registries,
        docker=docker,
    )
    docker_builder.login_private_registries()
    if docker_builder.check_image() and not nocache:
        # Image already built
        return docker_builder
    if not docker_builder.build(nocache=nocache):
        raise PolyaxonBuildException("The docker image could not be built.")
    return docker_builder


def build(
    context,
    destination,
    nocache,
    docker=None,
    credstore_env=None,
    registries=None,
    max_retries=3,
    sleep_interval=1,
):
    """Build necessary code for a job to run"""
    retry = 0
    is_done = False
    while retry < max_retries and not is_done:
        try:
            docker_builder = _build(
                context=context,
                destination=destination,
                docker=docker,
                nocache=nocache,
                credstore_env=credstore_env,
                registries=registries,
            )
            is_done = True
            return docker_builder
        except ReadTimeoutError:
            retry += 1
            time.sleep(sleep_interval)
    if not is_done:
        raise PolyaxonBuildException(
            "The docker image could not be built, client timed out."
        )


def push(destination, docker=None, max_retries=3, sleep_interval=1):
    docker_pusher = DockerPusher(destination=destination, docker=docker)
    retry = 0
    is_done = False
    while retry < max_retries and not is_done:
        try:
            if not docker_pusher.push():
                raise PolyaxonBuildException("The docker image could not be pushed.")
            else:
                is_done = True
        except ReadTimeoutError:
            retry += 1
            time.sleep(sleep_interval)

    if not is_done:
        raise PolyaxonBuildException(
            "The docker image could not be pushed, client timed out."
        )


def build_and_push(
    context,
    destination,
    nocache,
    credstore_env=None,
    registries=None,
    max_retries=3,
    sleep_interval=1,
):
    """Build necessary code for a job to run and push it."""
    # Build the image
    docker_builder = build(
        context=context,
        destination=destination,
        nocache=nocache,
        credstore_env=credstore_env,
        registries=registries,
        max_retries=max_retries,
        sleep_interval=sleep_interval,
    )
    push(
        destination=destination,
        docker=docker_builder.docker,
        max_retries=max_retries,
        sleep_interval=sleep_interval,
    )



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from marshmallow import fields

from polyaxon.schemas.base import BaseCamelSchema, BaseConfig


class CelerySchema(BaseCamelSchema):
    enabled = fields.Bool(allow_none=True)
    task_track_started = fields.Bool(allow_none=True)
    broker_pool_limit = fields.Int(allow_none=True)
    confirm_publish = fields.Bool(allow_none=True)
    worker_prefetch_multiplier = fields.Int(allow_none=True)
    worker_max_tasks_per_child = fields.Int(allow_none=True)
    worker_max_memory_per_child = fields.Int(allow_none=True)
    task_always_eager = fields.Bool(allow_none=True)

    @staticmethod
    def schema_config():
        return CeleryConfig


class CeleryConfig(BaseConfig):
    SCHEMA = CelerySchema
    REDUCED_ATTRIBUTES = [
        "enabled",
        "taskTrackStarted",
        "brokerPoolLimit",
        "confirmPublish",
        "workerPrefetchMultiplier",
        "workerMaxTasksPerChild",
        "workerMaxMemoryPerChild",
        "taskAlwaysEager",
    ]

    def __init__(
        self,
        enabled=None,
        task_track_started=None,
        broker_pool_limit=None,
        confirm_publish=None,
        worker_prefetch_multiplier=None,
        worker_max_tasks_per_child=None,
        worker_max_memory_per_child=None,
        task_always_eager=None,
    ):
        self.enabled = enabled
        self.task_track_started = task_track_started
        self.broker_pool_limit = broker_pool_limit
        self.confirm_publish = confirm_publish
        self.worker_prefetch_multiplier = worker_prefetch_multiplier
        self.worker_max_tasks_per_child = worker_max_tasks_per_child
        self.worker_max_memory_per_child = worker_max_memory_per_child
        self.task_always_eager = task_always_eager



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from polyaxon.k8s.custom_resources.operation import (
    get_cleaner_instance,
    get_cleaner_resource_name,
    get_notifier_instance,
    get_notifier_resource_name,
    get_tuner_instance,
    get_tuner_resource_name,
    get_watchdog_instance,
    get_watchdog_resource_name,
)
from polyaxon.polypod.compiler.converters.base import PlatformConverterMixin
from polyaxon.polypod.compiler.converters.job import JobConverter
from polyaxon.polypod.mixins import (
    CleanerMixin,
    NotifierMixin,
    TunerMixin,
    WatchDogMixin,
)


class NotifierConverter(NotifierMixin, JobConverter):
    def get_instance(self):
        return get_notifier_instance(
            owner=self.owner_name, project=self.project_name, run_uuid=self.run_uuid
        )

    def get_resource_name(self):
        return get_notifier_resource_name(self.run_uuid)


class CleanerConverter(CleanerMixin, JobConverter):
    def get_instance(self):
        return get_cleaner_instance(
            owner=self.owner_name, project=self.project_name, run_uuid=self.run_uuid
        )

    def get_resource_name(self):
        return get_cleaner_resource_name(self.run_uuid)


class TunerConverter(TunerMixin, JobConverter):
    def get_instance(self):
        return get_tuner_instance(
            owner=self.owner_name, project=self.project_name, run_uuid=self.run_uuid
        )

    def get_resource_name(self):
        return get_tuner_resource_name(self.run_uuid)


class WatchDogConverter(WatchDogMixin, JobConverter):
    def get_instance(self):
        return get_watchdog_instance(
            owner=self.owner_name, project=self.project_name, run_uuid=self.run_uuid
        )

    def get_resource_name(self):
        return get_watchdog_resource_name(self.run_uuid)


class PlatformNotifierConverter(PlatformConverterMixin, NotifierConverter):
    pass


class PlatformCleanerConverter(PlatformConverterMixin, CleanerConverter):
    pass


class PlatformTunerConverter(PlatformConverterMixin, TunerConverter):
    pass


class PlatformWatchDogConverter(PlatformConverterMixin, WatchDogConverter):
    pass



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import pytest
import uuid

from mock import patch

from polyaxon import settings
from polyaxon.env_vars.keys import POLYAXON_KEYS_RUN_INSTANCE
from polyaxon.exceptions import PolyaxonContainerException
from polyaxon.init.auth import create_auth_context
from tests.utils import BaseTestCase


@pytest.mark.init_mark
class TestInitAuth(BaseTestCase):
    def test_raise_if_env_var_not_found(self):
        with self.assertRaises(PolyaxonContainerException):
            create_auth_context()

    def test_raise_if_env_var_not_correct(self):
        os.environ[POLYAXON_KEYS_RUN_INSTANCE] = "foo"
        with self.assertRaises(PolyaxonContainerException):
            create_auth_context()
        del os.environ[POLYAXON_KEYS_RUN_INSTANCE]

    @patch("polyaxon_sdk.RunsV1Api.impersonate_token")
    @patch("polyaxon_sdk.UsersV1Api.get_user")
    @patch("polyaxon.client.impersonate.create_context_auth")
    def test_init_auth(self, create_context, get_user, impersonate_token):
        settings.CLIENT_CONFIG.is_managed = True
        os.environ[POLYAXON_KEYS_RUN_INSTANCE] = "owner.project.runs.{}".format(
            uuid.uuid4().hex
        )
        create_auth_context()
        assert impersonate_token.call_count == 1
        assert create_context.call_count == 1
        assert get_user.call_count == 1
        del os.environ[POLYAXON_KEYS_RUN_INSTANCE]
        settings.CLIENT_CONFIG.is_managed = None



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from polyaxon.auxiliaries.cleaner import get_default_cleaner_container
from polyaxon.auxiliaries.init import (
    PolyaxonInitContainerSchema,
    V1PolyaxonInitContainer,
    get_default_init_container,
    get_init_resources,
)
from polyaxon.auxiliaries.notifier import get_default_notification_container
from polyaxon.auxiliaries.sidecar import (
    PolyaxonSidecarContainerSchema,
    V1PolyaxonSidecarContainer,
    get_default_sidecar_container,
    get_sidecar_resources,
)
from polyaxon.auxiliaries.tuner import get_default_tuner_container



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from collections.abc import Mapping

from marshmallow import fields


class DictOrStr(fields.Str):
    default_error_messages = {
        "invalid": "Not a valid string or dict.",
        "invalid_utf8": "Not a valid utf-8 string.",
    }

    def _serialize(self, value, attr, obj, **kwargs):
        if isinstance(value, Mapping):
            return value

        return super()._serialize(value=value, attr=attr, obj=obj)

    def _deserialize(self, value, attr, data, **kwargs):
        if isinstance(value, Mapping):
            return value

        return super()._deserialize(value=value, attr=attr, data=data)



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pytest

from marshmallow import ValidationError

from polyaxon.polyflow.schedules import (
    ScheduleSchema,
    V1CronSchedule,
    V1DateTimeSchedule,
    V1IntervalSchedule,
)
from polyaxon.utils.tz_utils import now
from tests.utils import BaseTestCase


@pytest.mark.polyflow_mark
class TestScheduleConfigs(BaseTestCase):
    def test_interval_schedule(self):
        config_dict = {"frequency": 2, "startAt": "foo"}
        with self.assertRaises(ValidationError):
            V1IntervalSchedule.from_dict(config_dict)

        config_dict = {"frequency": "foo", "startAt": now().isoformat()}
        with self.assertRaises(ValidationError):
            V1IntervalSchedule.from_dict(config_dict)

        config_dict = {
            "kind": "cron",
            "frequency": 2,
            "startAt": now().isoformat(),
            "endAt": now().isoformat(),
        }
        with self.assertRaises(ValidationError):
            V1IntervalSchedule.from_dict(config_dict)

        config_dict = {"frequency": 2, "startAt": now().isoformat()}
        V1IntervalSchedule.from_dict(config_dict)

        config_dict = {
            "frequency": 2,
            "startAt": now().isoformat(),
            "endAt": now().isoformat(),
        }
        V1IntervalSchedule.from_dict(config_dict)

        config_dict = {
            "kind": "interval",
            "frequency": 2,
            "startAt": now().isoformat(),
            "endAt": now().isoformat(),
            "maxRuns": 123,
            "dependsOnPast": False,
        }
        V1IntervalSchedule.from_dict(config_dict)

    def test_cron_schedule(self):
        config_dict = {"cron": 2, "startAt": "foo"}
        with self.assertRaises(ValidationError):
            V1CronSchedule.from_dict(config_dict)

        config_dict = {
            "kind": "interval",
            "cron": "0 0 * * *",
            "startAt": now().isoformat(),
            "endAt": now().isoformat(),
        }
        with self.assertRaises(ValidationError):
            V1CronSchedule.from_dict(config_dict)

        config_dict = {"cron": "0 0 * * *"}
        V1CronSchedule.from_dict(config_dict)

        config_dict = {
            "cron": "0 0 * * *",
            "startAt": now().isoformat(),
            "endAt": now().isoformat(),
        }
        V1CronSchedule.from_dict(config_dict)

        config_dict = {
            "kind": "cron",
            "cron": "0 0 * * *",
            "startAt": now().isoformat(),
            "endAt": now().isoformat(),
            "maxRuns": 123,
            "dependsOnPast": False,
        }
        V1CronSchedule.from_dict(config_dict)

    def test_date_schedule(self):
        config_dict = {"startAt": "foo"}
        with self.assertRaises(ValidationError):
            V1DateTimeSchedule.from_dict(config_dict)

        config_dict = {"kind": "datetime", "startAt": now().isoformat()}
        V1DateTimeSchedule.from_dict(config_dict).to_dict()

    def test_schedule(self):
        configs = [
            {
                "kind": "interval",
                "frequency": 2,
                "startAt": now().isoformat(),
                "endAt": now().isoformat(),
                "dependsOnPast": False,
            },
            {
                "kind": "cron",
                "cron": "0 0 * * *",
                "startAt": now().isoformat(),
                "endAt": now().isoformat(),
                "dependsOnPast": True,
            },
            {
                "kind": "datetime",
                "startAt": "2010-01-01T00:00+00:00",
            },
        ]

        ScheduleSchema().load(configs, many=True)



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

from polyaxon.config_reader.manager import ConfigManager
from polyaxon.exceptions import PolyaxonSchemaError
from polyaxon.schemas.types import V1AuthType, V1UriType
from tests.utils import BaseTestCase


class TestConfigManager(BaseTestCase):
    def setUp(self):
        super().setUp()
        os.environ["FOO_BAR_KEY"] = "foo_bar"
        self.config = ConfigManager.read_configs(
            [os.environ, "tests/fixtures/parsing/configs/config_tests.json"]
        )

    def test_get_from_os_env(self):
        assert self.config.get_string("FOO_BAR_KEY") == "foo_bar"

    def test_reading_invalid_json_config_raises_error(self):
        with self.assertRaises(PolyaxonSchemaError):
            ConfigManager.read_configs(
                ["tests/fixtures/parsing/configs/invalid_config_tests.json"]
            )

    def test_reading_invalid_yaml_config_raises_error(self):
        with self.assertRaises(PolyaxonSchemaError):
            ConfigManager.read_configs(
                ["tests/fixtures/parsing/configs/invalid_config_tests.yaml"]
            )

    def test_get_boolean(self):
        value = self.config.get_boolean("bool_key_1")
        self.assertEqual(value, True)

        value = self.config.get_boolean("bool_key_2")
        self.assertEqual(value, True)

        value = self.config.get_boolean("bool_key_3")
        self.assertEqual(value, False)

        value = self.config.get_boolean("bool_key_4")
        self.assertEqual(value, False)

        value = self.config.get_boolean("bool_list_key_1", is_list=True)
        self.assertEqual(value, [False, False, True, True, True, False])

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_boolean("bool_error_key_1")

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_boolean("bool_error_key_2")

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_boolean("bool_error_key_3")

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_boolean("bool_error_key_4")

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_boolean("bool_error_key_5")

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_boolean("bool_list_key_1")

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_boolean("bool_list_error_key_2", is_list=True)

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_boolean("bool_list_error_key_1", is_list=True)

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_boolean("bool_key_1", is_list=True)

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_boolean("bool_key_2", is_list=True)

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_boolean("bool_non_existing_key")

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_boolean("bool_non_existing_key", is_list=True)

        self.assertEqual(
            self.config.get_boolean("bool_non_existing_key", is_optional=True), None
        )
        self.assertEqual(
            self.config.get_boolean(
                "bool_non_existing_key", is_optional=True, default=True
            ),
            True,
        )

        self.assertEqual(
            self.config.get_boolean(
                "bool_non_existing_key", is_list=True, is_optional=True
            ),
            None,
        )
        self.assertEqual(
            self.config.get_boolean(
                "bool_non_existing_key",
                is_list=True,
                is_optional=True,
                default=[True, False],
            ),
            [True, False],
        )

    def test_get_int(self):
        value = self.config.get_int("int_key_1")
        self.assertEqual(value, 123)

        value = self.config.get_int("int_key_2")
        self.assertEqual(value, 123)

        value = self.config.get_int("int_list_key_1", is_list=True)
        self.assertEqual(value, [123, 124, 125, 125])

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_int("int_error_key_1")

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_int("int_error_key_2")

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_int("int_error_key_3")

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_int("int_list_key_1")

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_int("int_list_error_key_1", is_list=True)

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_int("int_list_error_key_2", is_list=True)

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_int("int_list_error_key_3", is_list=True)

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_int("int_key_1", is_list=True)

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_int("int_key_2", is_list=True)

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_int("int_non_existing_key")

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_int("int_non_existing_key", is_list=True)

        self.assertEqual(
            self.config.get_int("int_non_existing_key", is_optional=True), None
        )
        self.assertEqual(
            self.config.get_int("int_non_existing_key", is_optional=True, default=34),
            34,
        )

        self.assertEqual(
            self.config.get_int("int_non_existing_key", is_list=True, is_optional=True),
            None,
        )
        self.assertEqual(
            self.config.get_int(
                "int_non_existing_key", is_list=True, is_optional=True, default=[34, 1]
            ),
            [34, 1],
        )

    def test_get_float(self):
        value = self.config.get_float("float_key_1")
        self.assertEqual(value, 1.23)

        value = self.config.get_float("float_key_2")
        self.assertEqual(value, 1.23)

        value = self.config.get_float("float_key_3")
        self.assertEqual(value, 123)

        value = self.config.get_float("float_list_key_1", is_list=True)
        self.assertEqual(value, [1.23, 13.3, 4.4, 555.0, 66.0])

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_float("float_error_key_1")

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_float("float_error_key_2")

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_float("float_error_key_3")

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_float("float_error_key_4")

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_float("float_list_key_1")

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_float("float_list_error_key_1", is_list=True)

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_float("float_list_error_key_2", is_list=True)

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_float("float_list_error_key_3", is_list=True)

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_float("float_key_1", is_list=True)

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_float("float_key_2", is_list=True)

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_float("float_non_existing_key")

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_float("float_non_existing_key", is_list=True)

        self.assertEqual(
            self.config.get_float("float_non_existing_key", is_optional=True), None
        )
        self.assertEqual(
            self.config.get_float(
                "float_non_existing_key", is_optional=True, default=3.4
            ),
            3.4,
        )

        self.assertEqual(
            self.config.get_float(
                "float_non_existing_key", is_list=True, is_optional=True
            ),
            None,
        )
        self.assertEqual(
            self.config.get_float(
                "float_non_existing_key",
                is_list=True,
                is_optional=True,
                default=[3.4, 1.2],
            ),
            [3.4, 1.2],
        )

    def test_get_string(self):
        value = self.config.get_string("string_key_1")
        self.assertEqual(value, "123")

        value = self.config.get_string("string_key_2")
        self.assertEqual(value, "1.23")

        value = self.config.get_string("string_key_3")
        self.assertEqual(value, "foo")

        value = self.config.get_string("string_key_4")
        self.assertEqual(value, "")

        value = self.config.get_string("string_list_key_1", is_list=True)
        self.assertEqual(value, ["123", "1.23", "foo", ""])

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_string("string_error_key_1")

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_string("string_error_key_2")

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_string("string_error_key_3")

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_string("string_error_key_4")

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_string("string_list_key_1")

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_string("string_list_error_key_1", is_list=True)

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_string("string_list_error_key_2", is_list=True)

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_string("string_list_error_key_3", is_list=True)

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_string("string_list_error_key_4", is_list=True)

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_string("string_key_3", is_list=True)

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_string("string_key_4", is_list=True)

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_string("string_non_existing_key")

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_string("string_non_existing_key", is_list=True)

        self.assertEqual(
            self.config.get_string("string_non_existing_key", is_optional=True), None
        )
        self.assertEqual(
            self.config.get_string(
                "string_non_existing_key", is_optional=True, default="foo"
            ),
            "foo",
        )

        self.assertEqual(
            self.config.get_string(
                "string_non_existing_key", is_list=True, is_optional=True
            ),
            None,
        )
        self.assertEqual(
            self.config.get_string(
                "string_non_existing_key",
                is_list=True,
                is_optional=True,
                default=["foo", "bar"],
            ),
            ["foo", "bar"],
        )

    def test_get_dict(self):
        value = self.config.get_dict("dict_key_1")
        self.assertEqual(value, {"key1": "foo", "key2": 2, "key3": False, "key4": "1"})

        value = self.config.get_dict("dict_list_key_1", is_list=True)
        self.assertEqual(
            value,
            [
                {"key1": "foo", "key2": 2, "key3": False, "key4": "1"},
                {"key3": True, "key4": "2"},
                {"key1": False, "key2": "3"},
            ],
        )

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_dict("dict_error_key_1")

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_dict("dict_error_key_2")

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_dict("dict_error_key_3")

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_dict("dict_error_key_4")

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_dict("dict_list_key_1")

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_dict("dict_list_error_key_1", is_list=True)

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_dict("dict_list_error_key_2", is_list=True)

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_dict("dict_list_error_key_3", is_list=True)

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_dict("dict_list_error_key_4", is_list=True)

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_dict("dict_key_1", is_list=True)

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_dict("dict_non_existing_key")

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_dict("dict_non_existing_key", is_list=True)

        self.assertEqual(
            self.config.get_dict("dict_non_existing_key", is_optional=True), None
        )
        self.assertEqual(
            self.config.get_dict(
                "dict_non_existing_key", is_optional=True, default={"foo": "bar"}
            ),
            {"foo": "bar"},
        )

        self.assertEqual(
            self.config.get_dict(
                "dict_non_existing_key", is_list=True, is_optional=True
            ),
            None,
        )
        self.assertEqual(
            self.config.get_dict(
                "dict_non_existing_key",
                is_list=True,
                is_optional=True,
                default=[{"foo": "bar"}, {"foo": "boo"}],
            ),
            [{"foo": "bar"}, {"foo": "boo"}],
        )

    def test_get_uri(self):
        value = self.config.get_uri("uri_key_1")
        self.assertEqual(value, V1UriType("user", "pass", "siteweb.ca"))

        value = self.config.get_uri("uri_key_2")
        self.assertEqual(value, V1UriType("user2", "pass", "localhost:8080"))

        value = self.config.get_uri("uri_key_3")
        self.assertEqual(value, V1UriType("user2", "pass", "https://quay.io"))

        value = self.config.get_uri("uri_list_key_1", is_list=True)
        self.assertEqual(
            value,
            [
                V1UriType("user", "pass", "siteweb.ca"),
                V1UriType("user2", "pass", "localhost:8080"),
                V1UriType("user2", "pass", "https://quay.io"),
            ],
        )

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_uri("uri_error_key_1")

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_uri("uri_error_key_2")

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_uri("uri_error_key_3")

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_uri("uri_error_key_4")

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_uri("uri_list_key_1")

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_uri("uri_list_error_key_1", is_list=True)

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_uri("uri_list_error_key_2", is_list=True)

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_uri("uri_list_error_key_3", is_list=True)

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_uri("uri_list_error_key_4", is_list=True)

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_uri("uri_key_1", is_list=True)

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_uri("uri_non_existing_key")

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_uri("uri_non_existing_key", is_list=True)

        self.assertEqual(
            self.config.get_uri("uri_non_existing_key", is_optional=True), None
        )
        self.assertEqual(
            self.config.get_uri(
                "uri_non_existing_key",
                is_optional=True,
                default=V1UriType("user2", "pass", "localhost:8080"),
            ),
            V1UriType("user2", "pass", "localhost:8080"),
        )

        self.assertEqual(
            self.config.get_uri("uri_non_existing_key", is_list=True, is_optional=True),
            None,
        )

        self.assertEqual(
            self.config.get_uri(
                "uri_non_existing_key",
                is_list=True,
                is_optional=True,
                default=[
                    V1UriType("user", "pass", "siteweb.ca"),
                    V1UriType("user2", "pass", "localhost:8080"),
                ],
            ),
            [
                V1UriType("user", "pass", "siteweb.ca"),
                V1UriType("user2", "pass", "localhost:8080"),
            ],
        )

    def test_get_auth(self):
        value = self.config.get_auth("auth_key_1")
        self.assertEqual(value, V1AuthType("user", "pass"))

        value = self.config.get_auth("auth_list_key_1", is_list=True)
        self.assertEqual(
            value, [V1AuthType("user", "pass"), V1AuthType("user2", "pass")]
        )

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_auth("auth_error_key_1")

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_auth("auth_error_key_2")

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_auth("auth_error_key_3")

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_auth("auth_error_key_4")

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_auth("auth_list_key_1")

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_auth("auth_list_error_key_1", is_list=True)

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_auth("auth_list_error_key_2", is_list=True)

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_auth("auth_list_error_key_3", is_list=True)

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_auth("auth_list_error_key_4", is_list=True)

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_auth("auth_key_1", is_list=True)

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_auth("auth_non_existing_key")

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_auth("auth_non_existing_key", is_list=True)

        self.assertEqual(
            self.config.get_auth("auth_non_existing_key", is_optional=True), None
        )
        self.assertEqual(
            self.config.get_auth(
                "auth_non_existing_key",
                is_optional=True,
                default=V1AuthType("user2", "pass"),
            ),
            V1AuthType("user2", "pass"),
        )

        self.assertEqual(
            self.config.get_auth(
                "auth_non_existing_key", is_list=True, is_optional=True
            ),
            None,
        )
        self.assertEqual(
            self.config.get_auth(
                "auth_non_existing_key",
                is_list=True,
                is_optional=True,
                default=[V1AuthType("user", "pass"), V1AuthType("user2", "pass")],
            ),
            [V1AuthType("user", "pass"), V1AuthType("user2", "pass")],
        )

    def test_get_list(self):
        value = self.config.get_list("list_key_1")
        self.assertEqual(value, ["user:pass@siteweb.ca", "'pp'", "0.1", "'foo'"])

        value = self.config.get_list("list_key_2")
        self.assertEqual(value, ["user1", "user2", "user3", "user4", "user5"])

        value = self.config.get_list("list_key_3")
        self.assertEqual(value, [False])

        value = self.config.get_list("list_key_4")
        self.assertEqual(value, ["foo"])

        value = self.config.get_list("list_key_5")
        self.assertEqual(value, [])

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_list("list_error_key_1")

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_list("list_error_key_2")

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_list("list_error_key_3")

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_list("list_error_key_4")

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_list("list_non_existing_key")

        self.assertEqual(
            self.config.get_list("list_non_existing_key", is_optional=True), None
        )
        self.assertEqual(
            self.config.get_list(
                "list_non_existing_key", is_optional=True, default=["foo"]
            ),
            ["foo"],
        )

    def test_get_dict_of_dicts(self):
        value = self.config.get_dict_of_dicts("dict_dicts_key_1")
        self.assertEqual(
            value, {"data1": {"mountPath": "/data/21", "existingClaim": "data-1-pvc"}}
        )

        value = self.config.get_dict_of_dicts("dict_dicts_key_2")
        self.assertEqual(
            value,
            {
                "outputs1": {
                    "mountPath": "/output/2",
                    "existingClaim": "outputs-1-pvc",
                },
                "outputs2": {"mountPath": "/output/2", "existingClaim": "output-2-pvc"},
            },
        )

        value = self.config.get_dict_of_dicts("dict_dicts_key_3")
        self.assertEqual(value, None)

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_dict_of_dicts("dict_dicts_error_key_1")

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_dict_of_dicts("dict_dicts_error_key_2")

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_dict_of_dicts("dict_dicts_error_key_3")

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_dict_of_dicts("dict_dicts_error_key_4")

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_dict_of_dicts("dict_dicts_error_key_5")

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_dict_of_dicts("dict_dicts_error_key_6")

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_dict_of_dicts("dict_dicts_error_key_7")

        with self.assertRaises(PolyaxonSchemaError):
            self.config.get_dict_of_dicts("dict_dicts_non_existing_key")

        self.assertEqual(
            self.config.get_dict_of_dicts(
                "dict_dicts_non_existing_key", is_optional=True
            ),
            None,
        )
        self.assertEqual(
            self.config.get_dict_of_dicts(
                "dict_dicts_non_existing_key", is_optional=True, default={}
            ),
            {},
        )



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import uuid

from polyaxon.deploy.operators.cmd_operator import CmdOperator


class ComposeOperator(CmdOperator):
    CMD = "docker-compose"

    @classmethod
    def params(cls, args):
        params = [cls.CMD] + args
        return params

    @classmethod
    def check(cls):
        command_exist = cls.execute(args=["version"])
        if not command_exist:
            return False
        return True

    @classmethod
    def execute(cls, args, stream=False):
        params = cls.params(args)
        return cls._execute(params=params, env=None, stream=stream)

    @staticmethod
    def _generate_worker_env(config):
        # pylint:disable=too-many-branches
        # pylint:disable=too-many-branches
        template = "{}={}"
        env = []
        if not config.worker:
            env.append(template.format("POLYAXON_WORKER_CONCURRENCY", 2))
            return env

        concurrency = config.worker.concurrency or 2
        env.append(template.format("POLYAXON_WORKER_CONCURRENCY", concurrency))
        if config.worker.celery:
            celery = config.worker.celery
            if celery.task_track_started:
                env.append(
                    template.format(
                        "POLYAXON_WORKER_CONCURRENCY", celery.task_track_started
                    )
                )
            if celery.broker_pool_limit:
                env.append(
                    template.format(
                        "POLYAXON_CELERY_BROKER_POOL_LIMIT", celery.broker_pool_limit
                    )
                )
            if celery.confirm_publish:
                env.append(
                    template.format(
                        "POLYAXON_CELERY_CONFIRM_PUBLISH", celery.confirm_publish
                    )
                )
            if celery.worker_prefetch_multiplier:
                env.append(
                    template.format(
                        "POLYAXON_CELERY_WORKER_PREFETCH_MULTIPLIER",
                        celery.worker_prefetch_multiplier,
                    )
                )
            if celery.worker_max_tasks_per_child:
                env.append(
                    template.format(
                        "POLYAXON_CELERY_WORKER_MAX_TASKS_PER_CHILD",
                        celery.worker_max_tasks_per_child,
                    )
                )
            if celery.worker_max_memory_per_child:
                env.append(
                    template.format(
                        "POLYAXON_CELERY_WORKER_MAX_MEMORY_PER_CHILD",
                        celery.worker_max_memory_per_child,
                    )
                )
            if celery.task_always_eager:
                env.append(
                    template.format(
                        "POLYAXON_CELERY_TASK_ALWAYS_EAGER", celery.task_always_eager
                    )
                )
        return env

    @staticmethod
    def _generate_postgres(config):
        template = "{}={}"
        env = []
        if config.postgresql and not config.postgresql.enabled:
            env.append(
                template.format(
                    "POLYAXON_DB_USER", config.external_services.postgresql.user
                )
            )
            env.append(
                template.format(
                    "POLYAXON_DB_PASSWORD", config.external_services.postgresql.password
                )
            )
            env.append(
                template.format(
                    "POLYAXON_DB_NAME", config.external_services.postgresql.database
                )
            )
            env.append(
                template.format(
                    "POLYAXON_DB_HOST", config.external_services.postgresql.host
                )
            )
        return env

    @staticmethod
    def _generate_intervals(config):
        template = "{}={}"
        env = []
        if config.intervals:
            env.append(
                template.format(
                    "POLYAXON_INTERVALS_RUNS_SCHEDULER", config.intervals.runs_scheduler
                )
            )
            env.append(
                template.format(
                    "POLYAXON_INTERVALS_OPERATIONS_DEFAULT_RETRY_DELAY",
                    config.intervals.operations_default_retry_delay,
                )
            )
            env.append(
                template.format(
                    "POLYAXON_INTERVALS_OPERATIONS_MAX_RETRY_DELAY",
                    config.intervals.operations_max_retry_delay,
                )
            )
        return env

    @staticmethod
    def _generate_email(config):
        template = "{}={}"
        env = []
        if config.email:
            env.append(template.format("POLYAXON_EMAIL_HOST", config.email.host))
            env.append(template.format("POLYAXON_EMAIL_PORT", config.email.port))
            env.append(template.format("POLYAXON_EMAIL_USE_TLS", config.email.use_tls))
            if config.email.host_user:
                env.append(
                    template.format("POLYAXON_EMAIL_HOST_USER", config.email.host_user)
                )
            if config.email.backend:
                env.append(
                    template.format("POLYAXON_EMAIL_BACKEND", config.email.backend)
                )
        return env

    @classmethod
    def generate_env(cls, config):
        # pylint:disable=too-many-statements
        # pylint:disable=too-many-branches
        template = "{}={}"
        env = [template.format("POLYAXON_DEPLOYMENT_TYPE", config.deployment_type)]
        if config.deployment_version:
            env.append(
                template.format("POLYAXON_CHART_VERSION", config.deployment_version)
            )
        if config.namespace:
            env.append(template.format("POLYAXON_K8S_NAMESPACE", config.namespace))
        env.append(
            template.format(
                "POLYAXON_SECRET_KEY", config.polyaxon_secret or uuid.uuid4().hex
            )
        )
        env.append(
            template.format(
                "POLYAXON_SECRET_INTERNAL_TOKEN",
                config.internal_token or uuid.uuid4().hex,
            )
        )
        if config.ssl and config.ssl.enabled and config.ssl.path:
            env.append(template.format("POLYAXON_SSL_ENABLED", config.ssl.enabled))
            env.append(template.format("POLYAXON_SSL_PATH", config.ssl.path))
        env.append(
            template.format(
                "POLYAXON_UI_ADMIN_ENABLED", config.ui.admin_enabled or False
            )
        )
        if config.timezone:
            env.append(template.format("POLYAXON_TIMEZONE", config.timezone or False))
        env += cls._generate_worker_env(config)
        env += cls._generate_email(config)

        if config.host_name:
            env.append(template.format("POLYAXON_HOST", config.host_name))
        if config.allowed_hosts:
            env.append(template.format("POLYAXON_ALLOWED_HOSTS", config.allowed_hosts))

        env += cls._generate_intervals(config)

        if config.log_level:
            env.append(template.format("POLYAXON_LOG_LEVEL", config.log_level))
        env += cls._generate_postgres(config)

        return "\n".join(env)



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

NAME = "polyaxon"
VERSION = "1.3.0-rc0"
SCHEMA_VERSION = 1.1
DESC = "Command Line Interface (CLI) for Polyaxon."
URL = "https://github.com/polyaxon/polyaxon"
AUTHOR = "Polyaxon, Inc."
EMAIL = "contact@polyaxon.com"
LICENSE = "Apache 2.0"



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

from typing import List, Optional

from polyaxon.auxiliaries import V1PolyaxonInitContainer
from polyaxon.containers.contexts import (
    CONTEXT_MOUNT_ARTIFACTS,
    CONTEXT_MOUNT_ARTIFACTS_FORMAT,
)
from polyaxon.containers.names import (
    INIT_ARTIFACTS_CONTAINER_PREFIX,
    generate_container_name,
)
from polyaxon.exceptions import PolypodException
from polyaxon.k8s import k8s_schemas
from polyaxon.polypod.common import constants
from polyaxon.polypod.common.containers import patch_container
from polyaxon.polypod.common.env_vars import (
    get_connection_env_var,
    get_env_from_config_map,
    get_env_from_secret,
    get_items_from_config_map,
    get_items_from_secret,
)
from polyaxon.polypod.common.mounts import (
    get_connections_context_mount,
    get_mount_from_resource,
    get_mount_from_store,
)
from polyaxon.polypod.common.volumes import get_volume_name
from polyaxon.schemas.types import V1ArtifactsType, V1ConnectionType
from polyaxon.utils.list_utils import to_list


def get_or_create_args(path):
    return 'if [ ! -d "{path}" ]; then mkdir -m 0777 -p {path}; fi;'.format(path=path)


def cp_copy_args(path_from, path_to, is_file: bool) -> str:
    if is_file:
        return "if [ -f {path_from} ]; then cp {path_from} {path_to}; fi;".format(
            path_from=path_from, path_to=path_to
        )
    return (
        'if [ -d {path_from} ] && [ "$(ls -A {path_from})" ]; '
        "then cp -r {path_from}/* {path_to}; fi;".format(
            path_from=path_from, path_to=path_to
        )
    )


def cp_gcs_args(path_from: str, path_to: str, is_file: bool) -> str:
    file_flag = "--is-file" if is_file else ""
    return "polyaxon initializer gcs --path-from={} --path-to={} {};".format(
        path_from, path_to, file_flag
    )


def cp_s3_args(path_from: str, path_to: str, is_file: bool) -> str:
    file_flag = "--is-file" if is_file else ""
    return "polyaxon initializer s3 --path-from={} --path-to={} {};".format(
        path_from, path_to, file_flag
    )


def cp_wasb_args(path_from: str, path_to: str, is_file: bool) -> str:
    file_flag = "--is-file" if is_file else ""
    return "polyaxon initializer wasb --path-from={} --path-to={} {};".format(
        path_from, path_to, file_flag
    )


def get_volume_args(
    store: V1ConnectionType, mount_path: str, artifacts: V1ArtifactsType
) -> str:
    files = []
    dirs = []
    if artifacts:
        files = artifacts.files or files
        dirs = artifacts.dirs or dirs
    # Default behavior is to pull all bucket
    if not files and not dirs:
        dirs = [""]
    args = []
    base_path_from = store.store_path

    def _copy():
        path_from = os.path.join(base_path_from, p)
        path_to = os.path.join(mount_path, p)

        # If we are initializing a file we need to create the base folder
        _p = os.path.split(p)[0]
        base_path_to = os.path.join(mount_path, _p)
        # We need to check that the path exists first
        args.append(get_or_create_args(path=base_path_to))

        # copy to context
        if store.is_wasb:
            args.append(
                cp_wasb_args(path_from=path_from, path_to=path_to, is_file=is_file)
            )
        elif store.is_s3:
            args.append(
                cp_s3_args(path_from=path_from, path_to=path_to, is_file=is_file)
            )
        elif store.is_gcs:
            args.append(
                cp_gcs_args(path_from=path_from, path_to=path_to, is_file=is_file)
            )
        else:
            args.append(
                cp_copy_args(path_from=path_from, path_to=path_to, is_file=is_file)
            )

    is_file = True
    for p in files:
        _copy()

    is_file = False
    for p in dirs:
        # We need to check that the path exists first
        _copy()

    return " ".join(args)


def get_base_store_container(
    container: Optional[k8s_schemas.V1Container],
    container_name: str,
    polyaxon_init: V1PolyaxonInitContainer,
    store: V1ConnectionType,
    env: List[k8s_schemas.V1EnvVar],
    env_from: List[k8s_schemas.V1EnvFromSource],
    volume_mounts: List[k8s_schemas.V1VolumeMount],
    args: List[str],
) -> Optional[k8s_schemas.V1Container]:
    env = env or []
    env_from = env_from or []
    volume_mounts = volume_mounts or []

    # Artifact store needs to allow init the contexts as well, so the store is not required
    if not store:
        raise PolypodException("Init store container requires a store")
    secret = None
    if store.is_bucket:
        secret = store.get_secret()
        volume_mounts = volume_mounts + to_list(
            get_mount_from_resource(resource=secret), check_none=True
        )
        env = env + to_list(get_items_from_secret(secret=secret), check_none=True)
        env_from = env_from + to_list(
            get_env_from_secret(secret=secret), check_none=True
        )
        env += to_list(
            get_connection_env_var(connection=store, secret=secret), check_none=True
        )

        config_map = store.get_config_map()
        volume_mounts = volume_mounts + to_list(
            get_mount_from_resource(resource=config_map), check_none=True
        )
        env = env + to_list(
            get_items_from_config_map(config_map=config_map), check_none=True
        )
        env_from = env_from + to_list(
            get_env_from_config_map(config_map=config_map), check_none=True
        )
    else:
        volume_mounts = volume_mounts + to_list(
            get_mount_from_store(store=store), check_none=True
        )
        env += to_list(
            get_connection_env_var(connection=store, secret=secret), check_none=True
        )

    return patch_container(
        container=container,
        name=container_name,
        image=polyaxon_init.get_image(),
        image_pull_policy=polyaxon_init.image_pull_policy,
        command=["/bin/sh", "-c"],
        args=args,
        env=env,
        env_from=env_from,
        resources=polyaxon_init.get_resources(),
        volume_mounts=volume_mounts,
    )


def get_store_container(
    polyaxon_init: V1PolyaxonInitContainer,
    connection: V1ConnectionType,
    artifacts: V1ArtifactsType,
    container: Optional[k8s_schemas.V1Container] = None,
    env: List[k8s_schemas.V1EnvVar] = None,
    mount_path: str = None,
    is_default_artifacts_store: bool = False,
) -> Optional[k8s_schemas.V1Container]:
    container_name = generate_container_name(
        INIT_ARTIFACTS_CONTAINER_PREFIX, connection.name
    )
    if not container:
        container = k8s_schemas.V1Container(name=container_name)

    volume_name = (
        get_volume_name(mount_path)
        if mount_path
        else constants.CONTEXT_VOLUME_ARTIFACTS
    )
    volume_mount_path = mount_path or (
        CONTEXT_MOUNT_ARTIFACTS_FORMAT.format(connection.name)
        if is_default_artifacts_store
        else CONTEXT_MOUNT_ARTIFACTS
    )
    volume_mounts = [
        get_connections_context_mount(name=volume_name, mount_path=volume_mount_path)
    ]
    mount_path = mount_path or CONTEXT_MOUNT_ARTIFACTS_FORMAT.format(connection.name)

    return get_base_store_container(
        container=container,
        container_name=container_name,
        polyaxon_init=polyaxon_init,
        store=connection,
        env=env,
        env_from=[],
        volume_mounts=volume_mounts,
        args=[
            get_volume_args(
                store=connection, mount_path=mount_path, artifacts=artifacts
            )
        ],
    )



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import re

from collections import namedtuple
from pathlib import PurePath

from polyaxon.logger import logger
from polyaxon.managers.base import BaseConfigManager
from polyaxon.utils import constants
from polyaxon.utils.path_utils import unix_style_path


class Pattern(namedtuple("Pattern", "pattern is_exclude re")):
    @staticmethod
    def create(pattern):
        if pattern[0:1] == "!":
            is_exclude = False
            pattern = pattern[1:]
        else:
            if pattern[0:1] == "\\":
                pattern = pattern[1:]
            is_exclude = True
        return Pattern(
            pattern=pattern,
            is_exclude=is_exclude,
            re=re.compile(translate(pattern), re.IGNORECASE),
        )

    def match(self, path):
        return bool(self.re.match(path))


def translate(pat):
    def _translate_segment():
        # pylint:disable=undefined-loop-variable
        if segment == "*":
            return "[^/]+"
        res = ""
        i, n = 0, len(segment)
        while i < n:
            c = segment[i : i + 1]
            i = i + 1
            if c == "*":
                res += "[^/]*"
            elif c == "?":
                res += "[^/]"
            elif c == "[":
                j = i
                if j < n and segment[j : j + 1] == "!":
                    j = j + 1
                if j < n and segment[j : j + 1] == "]":
                    j = j + 1
                while j < n and segment[j : j + 1] != "]":
                    j = j + 1
                if j >= n:
                    res += "\\["
                else:
                    stuff = segment[i:j].replace("\\", "\\\\")
                    i = j + 1
                    if stuff.startswith("!"):
                        stuff = "^" + stuff[1:]
                    elif stuff.startswith("^"):
                        stuff = "\\" + stuff
                    res += "[" + stuff + "]"
            else:
                res += re.escape(c)
        return res

    res = "(?ms)"

    if "/" not in pat[:-1]:
        res += "(.*/)?"

    if pat.startswith("**/"):
        pat = pat[2:]
        res += "(.*/)?"

    if pat.startswith("/"):
        pat = pat[1:]

    for i, segment in enumerate(pat.split("/")):
        if segment == "**":
            res += "(/.*)?"
            continue
        else:
            res += (re.escape("/") if i > 0 else "") + _translate_segment()

    if not pat.endswith("/"):
        res += "/?"

    return res + "\\Z"


class IgnoreConfigManager(BaseConfigManager):
    """Manages .polyaxonignore file in the current directory"""

    VISIBILITY = BaseConfigManager.VISIBILITY_LOCAL
    CONFIG_FILE_NAME = ".polyaxonignore"

    @staticmethod
    def _is_empty_or_comment(line):
        return not line or line.startswith("#")

    @staticmethod
    def _remove_trailing_spaces(line):
        """Remove trailing spaces unless they are quoted with a backslash."""
        while line.endswith(" ") and not line.endswith("\\ "):
            line = line[:-1]
        return line.replace("\\ ", " ")

    @classmethod
    def init_config(cls):
        cls.set_config(constants.DEFAULT_IGNORE_LIST, init=True)

    @classmethod
    def find_matching(cls, path, patterns):
        """Yield all matching patterns for path."""
        for pattern in patterns:
            if pattern.match(path):
                yield pattern

    @classmethod
    def is_ignored(cls, path, patterns):
        """Check whether a path is ignored. For directories, include a trailing slash."""
        status = None
        for pattern in cls.find_matching(path, patterns):
            status = pattern.is_exclude
        return status

    @classmethod
    def read_file(cls, ignore_file):
        for line in ignore_file:
            line = line.rstrip("\r\n")

            if cls._is_empty_or_comment(line):
                continue

            yield cls._remove_trailing_spaces(line)

    @classmethod
    def get_patterns(cls, ignore_file):
        return [Pattern.create(line) for line in cls.read_file(ignore_file)]

    @classmethod
    def get_config(cls):
        config_filepath = cls.get_config_filepath()

        if not os.path.isfile(config_filepath):
            return []

        with open(config_filepath) as ignore_file:
            return cls.get_patterns(ignore_file)

    @classmethod
    def get_unignored_filepaths(cls):
        config = cls.get_config()
        unignored_files = []

        for root, dirs, files in os.walk("."):
            logger.debug("Root:%s, Dirs:%s", root, dirs)

            if cls.is_ignored(unix_style_path(root), config):
                dirs[:] = []
                logger.debug("Ignoring directory : %s", root)
                continue

            for file_name in files:
                filepath = unix_style_path(os.path.join(root, file_name))
                if cls.is_ignored(filepath, config):
                    logger.debug("Ignoring file : %s", file_name)
                    continue

                unignored_files.append(os.path.join(root, file_name))

        return unignored_files

    @staticmethod
    def _matches_patterns(path, patterns):
        """Given a list of patterns, returns a if a path matches any pattern."""
        for glob in patterns:
            try:
                if PurePath(path).match(glob):
                    return True
            except TypeError:
                pass
        return False

    @classmethod
    def _ignore_path(cls, path, ignore_list=None, white_list=None):
        """Returns a whether a path should be ignored or not."""
        ignore_list = ignore_list or []
        white_list = white_list or []
        return cls._matches_patterns(path, ignore_list) and not cls._matches_patterns(
            path, white_list
        )

    @classmethod
    def get_value(cls, key):
        pass



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import math

from typing import Tuple

import polyaxon_sdk

from marshmallow import fields, validate

from polyaxon.polyflow.early_stopping import EarlyStoppingSchema
from polyaxon.polyflow.matrix.base import BaseSearchConfig
from polyaxon.polyflow.matrix.kinds import V1MatrixKind
from polyaxon.polyflow.matrix.params import HpParamSchema
from polyaxon.polyflow.optimization import (
    OptimizationMetricSchema,
    OptimizationResourceSchema,
)
from polyaxon.schemas.base import BaseCamelSchema
from polyaxon.schemas.fields.ref_or_obj import RefOrObject
from polyaxon.containers.names import MAIN_JOB_CONTAINER
from polyaxon.k8s import k8s_schemas
from polyaxon.schemas.fields.swagger import SwaggerField


class HyperbandSchema(BaseCamelSchema):
    kind = fields.Str(allow_none=True, validate=validate.Equal(V1MatrixKind.HYPERBAND))
    params = fields.Dict(
        keys=fields.Str(), values=fields.Nested(HpParamSchema), allow_none=True
    )
    max_iterations = RefOrObject(fields.Int(validate=validate.Range(min=1)))
    eta = RefOrObject(fields.Float(validate=validate.Range(min=0)))
    resource = fields.Nested(OptimizationResourceSchema)
    metric = fields.Nested(OptimizationMetricSchema)
    resume = RefOrObject(fields.Boolean(allow_none=True))
    seed = RefOrObject(fields.Int(allow_none=True))
    concurrency = fields.Int(allow_none=True)
    container = SwaggerField(
        cls=k8s_schemas.V1Container,
        defaults={"name": MAIN_JOB_CONTAINER},
        allow_none=True,
    )
    early_stopping = fields.Nested(EarlyStoppingSchema, many=True, allow_none=True)

    @staticmethod
    def schema_config():
        return V1Hyperband


class V1Hyperband(BaseSearchConfig, polyaxon_sdk.V1Hyperband):
    """Hyperband is a relatively new method for tuning iterative algorithms.
    It performs random sampling and attempts to gain an edge
    by using time spent optimizing in the best way.

    The algorithm tries a large number of random configurations/experiments,
    then decides which configurations to keep based on their progress.

    The way Hyperband is implemented, it creates several buckets,
    each bucket has a number of randomly generated hyperparameter configurations,
    each configuration uses a resource (e.g. number of steps, number of epochs, batch size, ...).

    To adapt the algorithm's maximum resource allocation, users can use `maxIterations`.

    After trying a number of configurations, it chooses the top `number of observation/eta`
    configurations and runs them using an increased `resource*eta` resource.
    At last, it chooses the best configuration it has found so far.

    The way Hyperband works is by discarding poor performing
    configurations leaving more resources for more promising configurations
    during the successive halving.

    In order to use Hyperband correctly, you must define a metric called
    `resource` that the algorithm will increase iteratively.

    Args:
        kind: string, should be equal to `hyperband`
        params: List[Dict[str,
            [params](/docs/automation/optimization-engine/params/#discrete-values)]]
        max_iterations: int
        eta: int
        resource: V1OptimizationResource
        metric: V1OptimizationMetric
        resume: bool, optional
        seed: int, optional
        concurrency: int, optional
        early_stopping: List[[EarlyStopping](/docs/automation/helpers/early-stopping)], optional

    ## YAML usage

    ```yaml
    >>> matrix:
    >>>   kind: hyperband
    >>>   concurrency:
    >>>   maxIterations:
    >>>   resource:
    >>>   metric:
    >>>   resume:
    >>>   params:
    >>>   seed:
    >>>   earlyStopping:
    ```

    ## Python usage

    ```python
    >>> from polyaxon import types
    >>> from polyaxon.polyflow import (
    >>>     V1Hyperband, V1HpLogSpace, V1HpChoice, V1FailureEarlyStopping, V1MetricEarlyStopping,
    >>>     V1OptimizationMetric, V1Optimization, V1OptimizationResource,
    >>> )
    >>> matrix = V1Hyperband(
    >>>   concurrency=20,
    >>>   params={"param1": V1HpLogSpace(...), "param2": V1HpChoice(...), ... },
    >>>   resume=True,
    >>>   metric=V1OptimizationMetric(name="loss", optimization=V1Optimization.MINIMIZE),
    >>>   resource=V1OptimizationResource(name="num_steps", type=types.INT),
    >>>   early_stopping=[V1FailureEarlyStopping(...), V1MetricEarlyStopping(...)]
    >>> )
    ```

    ## Fields

    ### kind

    The kind signals to the CLI, client, and other tools that this matrix is hyperband.

    If you are using the python client to create the mapping,
    this field is not required and is set by default.

    ```yaml
    >>> matrix:
    >>>   kind: hyperband
    ```

    ### params

    A dictionary of `key -> value generator`
    to generate the parameters.

    To learn about all possible
    [params generators](/docs/automation/optimization-engine/params/).

    > The parameters generated will be validated against
    the component's inputs/outputs definition to check that the values
    can be passed and have valid types.

    ```yaml
    >>> matrix:
    >>>   kind: hyperband
    >>>   params:
    >>>     param1:
    >>>        kind: ...
    >>>        value: ...
    >>>     param2:
    >>>        kind: ...
    >>>        value: ...
    ```

    ### maxIterations

    The algorithm's maximum resource allocation.

    ```yaml
    >>> matrix:
    >>>   kind: hyperband
    >>>   maxIterations: 81
    ```

    ### eta

    A parameter that tunes:
        * The downsampling factor: `number of observation/eta`
        * The resource increase factor: `resource*eta`

    ```yaml
    >>> matrix:
    >>>   kind: hyperband
    >>>   eta: 3
    ```

    ### resource

    The resource to optimize (should be an int or a float),
    the resource van be the number of steps or epochs,

    ```yaml
    >>> matrix:
    >>>   kind: hyperband
    >>>   resource:
    >>>     name: num_steps
    >>>     type: int
    ```

    ### metric

    The metric to optimize during the iterations,
    this is the metric that you want to maximize or minimize.

    ```yaml
    >>> matrix:
    >>>   kind: hyperband
    >>>   metric:
    >>>     name: loss
    >>>     optimization: minimize
    ```

    ### resume

    A flag to resume or restart the selected runs, default to false (restart)

    ```yaml
    >>> matrix:
    >>>   kind: hyperband
    >>>   resume: True
    ```

    ### concurrency

    An optional value to set the number of concurrent operations.

    <blockquote class="light">
    This value only makes sense if less or equal to the total number of possible runs.
    </blockquote>

    ```yaml
    >>> matrix:
    >>>   kind: random
    >>>   concurrency: 2
    ```

    For more details about concurrency management,
    please check the [concurrency section](/docs/automation/helpers/concurrency/).

     ### seed

    Since this algorithm uses random generators,
    if you want to control the seed for the random generator, you can pass a seed.

     ```yaml
    >>> matrix:
    >>>   kind: hyperband
    >>>   seed: 523
    ```

    ### earlyStopping

    A list of early stopping conditions to check for terminating
    all operations managed by the pipeline.
    If one of the early stopping conditions is met,
    a signal will be sent to terminate all running and pending operations.

    ```yaml
    >>> matrix:
    >>>   kind: hyperband
    >>>   earlyStopping: ...
    ```

    For more details please check the
    [early stopping section](/docs/automation/helpers/early-stopping/).

    ### container

    The container with the logic for creating new suggestions based on bayesian optimization,
    users can override this section to provide different resources requirements for the tuner.

    ```yaml
    >>> matrix:
    >>>   kind: hyperband
    >>>   container: ...
    ```

    ## Example

    This is an example of using hyperband for hyperparameter search:

    ```yaml
    >>> version: 1.1
    >>> kind: operation
    >>> matrix:
    >>>   kind: hyperband
    >>>   concurrency: 5
    >>>   maxIterations: 81
    >>>   eta: 3
    >>>   resource:
    >>>     name: num_steps
    >>>     type: int
    >>>   metric:
    >>>     name: loss
    >>>     optimization: minimize
    >>>   resume: False
    >>>   params:
    >>>     lr:
    >>>       kind: uniform
    >>>       value: [0, 0.9]
    >>>     dropout:
    >>>       kind: choice
    >>>       value: [0.25, 0.3]
    >>>     activation:
    >>>       kind: pchoice
    >>>       value: [[relu, 0.1], [sigmoid, 0.8]]
    >>>    early_stopping:
    >>>      - metric: accuracy
    >>>        value: 0.9
    >>>        optimization: maximize
    >>>      - metric: loss
    >>>        value: 0.05
    >>>        optimization: minimize
    >>> component:
    >>>   inputs:
    >>>     - name: batch_size
    >>>       type: int
    >>>       isOptional: true
    >>>       value: 128
    >>>     - name: logspace
    >>>       type: float
    >>>     - name: dropout
    >>>       type: float
    >>>   container:
    >>>     image: image:latest
    >>>     command: [python3, train.py]
    >>>     args: [
    >>>         "--batch-size={{ batch_size }}",
    >>>         "--lr={{ lr }}",
    >>>         "--dropout={{ dropout }}",
    >>>         "--activation={{ activation }}"
    >>>     ]
    ```

    In this example we allocate a maximum resources of `81`,
    our resource in this case is the `num_steps` which is of type `int` that we pass to our model.

    This is how the algorithm works with this config:

    |              | bucket=4   |                | bucket=3   |                 | bucket=2    |                | bucket=1   |                 | bucket=0   |                | # noqa
    |--------------|------------|----------------|------------|-----------------|-------------|----------------|------------|-----------------|------------|----------------| # noqa
    |iteration     |num configs |resource alloc  |num configs |resource alloc   |num configs  |resource alloc  |num configs |resource alloc   |num configs |resource alloc  | # noqa
    |0             |81          |1               |27          |3                |9            |9               |6           |27               |5           |             81 | # noqa
    |1             |27          |3               |9           |9                |3            |27              |2           |81               |            |                | # noqa
    |2             |9           |9               |3           |27               |1            |81              |            |                 |            |                | # noqa
    |3             |3           |27              |1           |81               |             |                |            |                 |            |                | # noqa
    |4             |1           |81              |            |                 |             |                |            |                 |            |                | # noqa
    """

    SCHEMA = HyperbandSchema
    IDENTIFIER = V1MatrixKind.HYPERBAND
    REDUCED_ATTRIBUTES = ["seed", "concurrency", "earlyStopping", "container"]

    def set_tuning_params(self):
        # Maximum iterations per configuration: max_iterations
        # Defines configuration downsampling/elimination rate (default = 3): eta
        # number of times to run hyperband (brackets)
        # i.e.  # of times to repeat the outer loops over the tradeoffs `s`
        self.s_max = int(math.log(self.max_iterations) / math.log(self.eta))
        self.B = (
            self.s_max + 1
        ) * self.max_iterations  # budget per bracket of successive halving

    def get_bracket(self, iteration):
        """This defines the bracket `s` in outerloop `for s in reversed(range(self.s_max))`."""
        return self.s_max - iteration

    def should_create_iteration(self, iteration, bracket_iteration):
        """Return a boolean to indicate if we need to reschedule another iteration."""
        bracket = self.get_bracket(iteration=iteration)
        if bracket_iteration < bracket:
            # The bracket is still processing
            return False

        # We can only reschedule if we can create a new bracket
        return self.get_bracket(iteration=iteration + 1) >= 0

    def get_num_runs_to_keep(self, num_runs, bracket_iteration):
        """Return the number of configs to keep and resume."""
        num_runs = num_runs * (self.eta ** -bracket_iteration)
        return int(num_runs / self.eta)

    def get_num_runs(self, bracket):
        # n: initial number of configs
        return int(
            math.ceil(
                (self.B / self.max_iterations) * (self.eta ** bracket) / (bracket + 1)
            )
        )

    def get_num_runs_to_keep_for_iteration(self, iteration, bracket_iteration):
        """Return the number of configs to keep for an iteration and iteration bracket.

        This is just util function around `get_num_runs_to_keep`
        """
        bracket = self.get_bracket(iteration=iteration)
        if bracket_iteration == bracket + 1:
            # End of loop `for bracket_iteration in range(bracket + 1):`
            return 0

        num_runs = self.get_num_runs(bracket=bracket)
        return self.get_num_runs_to_keep(
            num_runs=num_runs, bracket_iteration=bracket_iteration
        )

    def should_reduce_configs(self, iteration, bracket_iteration):
        """Return a boolean to indicate if we need to reschedule another bracket iteration."""
        num_runs_to_keep = self.get_num_runs_to_keep_for_iteration(
            iteration=iteration, bracket_iteration=bracket_iteration
        )
        return num_runs_to_keep > 0

    def create_iteration(
        self, iteration: int = None, bracket_iteration: int = 0
    ) -> Tuple[int, int]:
        """Create an iteration for hyperband."""
        if iteration is None:
            return 0, 0

        should_create_iteration = self.should_create_iteration(
            iteration=iteration,
            bracket_iteration=bracket_iteration,
        )
        should_reduce_configs = self.should_reduce_configs(
            iteration=iteration,
            bracket_iteration=bracket_iteration,
        )
        if should_create_iteration:
            iteration = iteration + 1
            bracket_iteration = 0
        elif should_reduce_configs:
            bracket_iteration = bracket_iteration + 1
        else:
            raise ValueError(
                "Hyperband create iteration failed, "
                "could not reschedule or reduce configs"
            )

        return iteration, bracket_iteration

    def should_reschedule(self, iteration, bracket_iteration):
        """Return a boolean to indicate if we need to reschedule another iteration."""
        return self.should_create_iteration(
            iteration=iteration, bracket_iteration=bracket_iteration
        ) or self.should_reduce_configs(
            iteration=iteration,
            bracket_iteration=bracket_iteration,
        )



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from marshmallow import fields

from polyaxon.schemas.base import BaseCamelSchema, BaseConfig


class UISchema(BaseCamelSchema):
    enabled = fields.Bool(allow_none=True)
    offline = fields.Bool(allow_none=True)
    static_url = fields.Str(allow_none=True)
    assets_version = fields.Str(allow_none=True)
    admin_enabled = fields.Bool(allow_none=True)

    @staticmethod
    def schema_config():
        return UIConfig


class UIConfig(BaseConfig):
    SCHEMA = UISchema
    REDUCED_ATTRIBUTES = [
        "enabled",
        "jsOffline",
        "staticUrl",
        "assetsVersion",
        "adminEnabled",
    ]

    def __init__(
        self,
        enabled=None,
        offline=None,
        static_url=None,
        assets_version=None,
        admin_enabled=None,
    ):
        self.enabled = enabled
        self.offline = offline
        self.assets_version = assets_version
        self.static_url = static_url
        self.admin_enabled = admin_enabled



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from polyaxon.config_reader.spec import ConfigSpec
from polyaxon.exceptions import PolyaxonSchemaError
from tests.utils import BaseTestCase


class TestReader(BaseTestCase):
    def test_reads_yaml_files(self):
        config = ConfigSpec.read_from("tests/fixtures/parsing/yaml_file.yml")
        assert config == {"x": 10, "y": 20, "foo": "bar", "type": "yaml"}

    def test_reads_json_files(self):
        config = ConfigSpec.read_from("tests/fixtures/parsing/json_file.json")
        assert config == {"x": 1, "y": 2, "foo": "bar", "type": "json"}

    def test_reads_yaml_files_without_extension(self):
        config = ConfigSpec.read_from(
            ConfigSpec("tests/fixtures/parsing/yaml_file", config_type=".yml")
        )
        assert config == {"x": 10, "y": 20, "foo": "bar", "type": "yaml"}

    def test_reads_json_files_without_extension(self):
        config = ConfigSpec.read_from(
            ConfigSpec("tests/fixtures/parsing/json_file", config_type=".json")
        )
        assert config == {"x": 1, "y": 2, "foo": "bar", "type": "json"}

    def test_reads_non_existing_file(self):
        # Raises by default
        with self.assertRaises(PolyaxonSchemaError):
            ConfigSpec.read_from("tests/fixtures/parsing/no_file.yml")

        with self.assertRaises(PolyaxonSchemaError):
            ConfigSpec.read_from("tests/fixtures/parsing/no_file.json")

        with self.assertRaises(PolyaxonSchemaError):
            ConfigSpec.read_from(ConfigSpec("tests/fixtures/parsing/no_file"))

        with self.assertRaises(PolyaxonSchemaError):
            ConfigSpec.read_from(ConfigSpec("tests/fixtures/parsing/no_file.yml"))

        with self.assertRaises(PolyaxonSchemaError):
            ConfigSpec.read_from(ConfigSpec("tests/fixtures/parsing/no_file.json"))

        # Does not raise if set to ignore
        assert (
            ConfigSpec.read_from(
                ConfigSpec("tests/fixtures/parsing/no_file", check_if_exists=False)
            )
            == {}
        )

        assert (
            ConfigSpec.read_from(
                ConfigSpec("tests/fixtures/parsing/no_file.yml", check_if_exists=False)
            )
            == {}
        )

        assert (
            ConfigSpec.read_from(
                ConfigSpec("tests/fixtures/parsing/no_file.json", check_if_exists=False)
            )
            == {}
        )

    def test_reads_config_map(self):
        config = ConfigSpec.read_from([{"x": "y"}, {1: 2}, {"x": "override y"}])
        assert config == {"x": "override y", 1: 2}

        config = ConfigSpec.read_from(
            [
                {"x": "y"},
                {1: 2},
                {"x": "override y"},
                "tests/fixtures/parsing/yaml_file.yml",
                "tests/fixtures/parsing/json_file.json",
            ]
        )
        assert config == {"x": 1, "y": 2, 1: 2, "foo": "bar", "type": "json"}

    def test_reads_yaml_stream(self):
        stream = """---
        x: y
        1: 2
        """
        config = ConfigSpec.read_from(stream)
        assert config == {"x": "y", 1: 2}

    def test_reads_non_valid_yaml_stream(self):
        stream = ";sdfsd;sdff"
        with self.assertRaises(PolyaxonSchemaError):
            ConfigSpec.read_from(stream)

    def test_reads_json_stream(self):
        stream = """---
        {x: y, 1: 2}
        """
        config = ConfigSpec.read_from(stream)
        assert config is not None



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import os
import pytest
import tempfile

from datetime import timedelta

from polyaxon.env_vars.keys import POLYAXON_KEYS_INTERVALS_COMPATIBILITY_CHECK
from polyaxon.schemas.cli.checks_config import ChecksConfig
from polyaxon.utils.tz_utils import now
from tests.utils import BaseTestCase


@pytest.mark.schemas_mark
class TestChecksConfig(BaseTestCase):
    def setUp(self):
        super().setUp()
        self.filename = "{}/{}".format(tempfile.mkdtemp(), "config")

    def test_get_interval(self):
        config = ChecksConfig()
        assert config.get_interval(1) == 1
        assert config.get_interval(-1) == -1
        assert config.get_interval(-2) == -2
        assert config.get_interval() == ChecksConfig.INTERVAL
        os.environ[POLYAXON_KEYS_INTERVALS_COMPATIBILITY_CHECK] = "-1"
        assert config.get_interval() == -1
        os.environ[POLYAXON_KEYS_INTERVALS_COMPATIBILITY_CHECK] = "-2"
        assert config.get_interval() == ChecksConfig.INTERVAL
        os.environ[POLYAXON_KEYS_INTERVALS_COMPATIBILITY_CHECK] = "1"
        assert config.get_interval() == ChecksConfig.INTERVAL
        os.environ[POLYAXON_KEYS_INTERVALS_COMPATIBILITY_CHECK] = str(
            ChecksConfig.INTERVAL + 1
        )
        assert config.get_interval() == ChecksConfig.INTERVAL + 1

    def test_should_check(self):
        config = ChecksConfig(last_check=now())
        assert config.should_check(-1) is False
        assert config.should_check(0) is True

        config.last_check = now() - timedelta(seconds=10000)
        assert config.should_check(-1) is False
        assert config.should_check() is True
        assert config.should_check(100) is True
        assert config.should_check(100000) is False



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

from typing import Any, List, Optional, Set, Union

from marshmallow import ValidationError

from polyaxon.env_vars.keys import (
    POLYAXON_KEYS_CONNECTION_CONTEXT_PATH_FORMAT,
    POLYAXON_KEYS_CONNECTION_SCHEMA_FORMAT,
)
from polyaxon.logger import logger
from polyaxon.schemas.types import V1ConnectionType
from polyaxon.utils.path_utils import check_dirname_exists


def get_from_env(keys: Union[Set[str], List[str], str]) -> Any:
    """
    Returns a variable from one of the list of keys based on the os.env.
    Args:
        keys: list(str). list of keys to check in the environment

    Returns:
        str | None
    """
    keys = keys or []
    if not isinstance(keys, (list, tuple, set)):
        keys = [keys]
    for key in keys:
        value = os.environ.get(key)
        if value:
            if value.lower() == "true":
                return True
            if value.lower() == "false":
                return False
            return value
        # Prepend POLYAXON
        key = "POLYAXON_{}".format(key)
        value = os.environ.get(key)
        if value:
            return value

    return None


def get_from_path(context_path: str, keys: Union[Set[str], List[str], str]) -> Any:
    """
    Returns a variable from one of the list of keys based on a base path.
    Args:
        context_path: str, base path where to look for keys.
        keys: list(str). list of keys to check in the environment

    Returns:
        str | None
    """
    if not check_dirname_exists(context_path, is_dir=True):
        return None

    keys = keys or []
    if not isinstance(keys, (list, tuple)):
        keys = [keys]
    for key in keys:
        key_path = os.path.join(context_path, key)
        if not os.path.exists(key_path):
            return None
        with open(key_path) as f:
            value = f.read()
            if value:
                if value.lower() == "true":
                    return True
                if value.lower() == "false":
                    return False
                return value

    return None


def get_connection_context_path_env_name(name: str) -> str:
    return POLYAXON_KEYS_CONNECTION_CONTEXT_PATH_FORMAT.format(name.upper())


def get_connection_schema_env_name(name: str) -> str:
    return POLYAXON_KEYS_CONNECTION_SCHEMA_FORMAT.format(name.upper())


def get_connection_context_path(name: Optional[str]) -> Optional[str]:
    """Checks if a connection has a mount path exported by Polyaxon"""
    if not name:
        return None

    context_path = os.environ.get(get_connection_context_path_env_name(name))
    if not context_path:
        return None

    if not os.path.exists(context_path):
        logger.warning(
            "A connection path was found for {}, "
            "but a path the {} does not exist.".format(name, context_path)
        )
        return None
    return context_path


def get_connection_type(name: Optional[str]) -> Optional[V1ConnectionType]:
    """Checks if a connection has a mount path exported by Polyaxon"""
    if not name:
        return None

    spec = os.environ.get(get_connection_schema_env_name(name))
    if not spec:
        return None

    try:
        spec = V1ConnectionType.read(spec, config_type=".json")
    except ValidationError as e:
        logger.warning(
            "A connection spec was found for {}, "
            "but the reading the spec raised an error {}.".format(name, repr(e))
        )
        return None
    return spec


def read_keys(context_path: str, keys: List[str]) -> Any:
    """Returns a variable by checking first a context path and then in the environment."""
    keys = (
        {k.lower() for k in keys}
        | {k.upper() for k in keys}
        | {"".join(k.lower().split("_")) for k in keys}
    )
    if context_path:
        value = get_from_path(context_path=context_path, keys=keys)
        if value is not None:
            return value
    return get_from_env(keys)



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
import os
import requests
import tarfile

from requests_toolbelt import MultipartEncoder, MultipartEncoderMonitor

from polyaxon import settings
from polyaxon.client.transport.utils import Bar, progress_bar
from polyaxon.exceptions import (
    HTTP_ERROR_MESSAGES_MAPPING,
    PolyaxonClientException,
    PolyaxonShouldExitError,
)
from polyaxon.logger import logger
from polyaxon.utils.list_utils import to_list


class HttpTransportMixin:
    """HTTP operations transport."""

    @property
    def session(self):
        if not hasattr(self, "_session"):
            self._session = requests.Session()
        return self._session

    @staticmethod
    def create_progress_callback(encoder):
        encoder_len = encoder.len
        callback_bar = Bar(expected_size=encoder_len, filled_char="=")

        def callback(monitor):
            callback_bar.show(monitor.bytes_read)

        return callback, callback_bar

    @staticmethod
    def format_sizeof(num, suffix="B"):
        """
        Print in human friendly format
        """
        for unit in ["", "Ki", "Mi", "Gi", "Ti", "Pi", "Ei", "Zi"]:
            if abs(num) < 1024.0:
                return "%3.1f%s%s" % (num, unit, suffix)
            num /= 1024.0
        return "%.1f%s%s" % (num, "Yi", suffix)

    def request(
        self,
        method,
        url,
        params=None,
        data=None,
        files=None,
        json=None,  # noqa
        timeout=None,
        headers=None,
        session=None,
    ):
        """Send a request with the given data as json to the given URL.

        Args:
            method: HTTP method
            url: The url to send the request to.
            params: Dictionary of values to format url.
            data:
            files:
            json:
            timeout:
            headers: extra headers to add to the request.

        Returns:
            Request response if successful, Exception otherwise.

        Raises:
            PolyaxonHTTPError or one of its subclasses.
        """
        logger.debug(
            "Starting request to url: %s with params: %s, data: %s", url, params, data
        )

        request_headers = self._get_headers(headers=headers)
        timeout = timeout if timeout is not None else settings.LONG_REQUEST_TIMEOUT
        session = session or self.session

        try:
            response = session.request(
                method,
                url,
                params=params,
                data=data,
                json=json,
                headers=request_headers,
                files=files,
                timeout=timeout,
                verify=self.config.verify_ssl,
            )
        except (
            requests.exceptions.RequestException,
            requests.exceptions.Timeout,
            requests.exceptions.HTTPError,
        ) as exception:
            try:
                logger.debug("Exception: %s", exception, exc_info=True)
            except TypeError:
                pass
            raise PolyaxonShouldExitError(
                "Error connecting to Polyaxon server on `{}`.\n"
                "An Error `{}` occurred.\n"
                "Check your host and ports configuration "
                "and your internet connection.".format(url, exception)
            )

        logger.debug(
            "Response Content: %s, Headers: %s", response.content, response.headers
        )
        self.check_response_status(response, url)
        return response

    def upload(
        self,
        url,
        files,
        files_size,
        params=None,
        json_data=None,
        timeout=None,
        headers=None,
        session=None,
    ):
        upload_size_max = os.environ.get("POLYAXON_UPLOAD_SIZE_MAX", 1024 * 1024 * 10)
        upload_size_warn = os.environ.get("POLYAXON_UPLOAD_SIZE_MAX", 1024 * 1024 * 150)
        if files_size > upload_size_warn:
            logger.warning(
                "You are uploading %s, there's a hard limit of %s.\n"
                "If you have data files in the current directory, "
                "please make sure to add them to .polyaxonignore or "
                "add them directly to your data volume, or upload them "
                "separately using `polyaxon data` command and remove them from here.\n",
                self.format_sizeof(files_size),
                self.format_sizeof(upload_size_max),
            )

        if files_size > upload_size_max:
            raise PolyaxonShouldExitError(
                "Files too large to sync, please keep it under {}.\n"
                "If you have data files in the current directory, "
                "please add them directly to your data volume, or upload them "
                "separately using `polyaxon data` command and remove them from here.\n".format(
                    self.format_sizeof(upload_size_max)
                )
            )

        files = to_list(files)
        if json_data:
            files.append(("json", json.dumps(json_data)))

        multipart_encoder = MultipartEncoder(fields=files)
        request_headers = headers or {}
        request_headers.update({"Content-Type": multipart_encoder.content_type})

        # Attach progress bar
        progress_callback, callback_bar = self.create_progress_callback(
            multipart_encoder
        )
        multipart_encoder_monitor = MultipartEncoderMonitor(
            multipart_encoder, progress_callback
        )

        timeout = timeout if timeout is not None else settings.LONG_REQUEST_TIMEOUT

        try:
            response = self.put(
                url=url,
                params=params,
                data=multipart_encoder_monitor,
                headers=request_headers,
                timeout=timeout,
                session=session,
            )
        finally:
            # always make sure we clear the console
            callback_bar.done()

        return response

    def download(
        self,
        url,
        filename,
        params=None,
        headers=None,
        timeout=None,
        session=None,
        untar=False,
        delete_tar=True,
        extract_path=None,
    ):
        """
        Download the file from the given url at the current path
        """
        # pylint:disable=too-many-branches
        logger.debug("Downloading files from url: %s", url)

        request_headers = self._get_headers(headers=headers)
        timeout = timeout if timeout is not None else settings.LONG_REQUEST_TIMEOUT
        session = session or self.session

        try:
            response = session.get(
                url,
                params=params,
                headers=request_headers,
                timeout=timeout,
                stream=True,
            )
            self.check_response_status(response, url)
            with open(filename, "wb") as f:
                # chunk mode response doesn't have content-length so we are
                # using a custom header here
                content_length = response.headers.get("x-polyaxon-content-length")
                if not content_length:
                    content_length = response.headers.get("content-length")
                if content_length:
                    for chunk in progress_bar(
                        response.iter_content(chunk_size=1024),
                        expected_size=(int(content_length) / 1024) + 1,
                    ):
                        if chunk:
                            f.write(chunk)
                else:
                    for chunk in response.iter_content(chunk_size=1024):
                        if chunk:
                            f.write(chunk)

            if untar:
                filename = self.untar(
                    filename=filename, delete_tar=delete_tar, extract_path=extract_path
                )
            return filename
        except (
            requests.exceptions.ConnectionError,
            requests.exceptions.RequestException,
            requests.exceptions.Timeout,
            requests.exceptions.HTTPError,
        ) as exception:
            try:
                logger.debug("Exception: %s", exception)
            except TypeError:
                pass

            raise PolyaxonShouldExitError(
                "Error connecting to Polyaxon server on `{}`.\n"
                "An Error `{}` occurred.\n"
                "Check your host and ports configuration "
                "and your internet connection.".format(url, exception)
            )

    def untar(self, filename, delete_tar=True, extract_path=None):
        extract_path = extract_path or "."
        logger.info("Untarring the contents of the file ...")
        tar = tarfile.open(filename)
        tar.extractall(extract_path)
        tar.close()
        if delete_tar:
            logger.info("Cleaning up the tar file ...")
            os.remove(filename)
        return filename

    def check_response_status(self, response, endpoint):
        """Check if response is successful. Else raise Exception."""

        if 200 <= response.status_code < 300:
            return response

        try:
            logger.error(
                "Request to %s failed with status code %s. \n" "Reason: %s",
                endpoint,
                response.status_code,
                response.text,
            )
        except TypeError:
            logger.error("Request to %s failed with status code", endpoint)

        raise PolyaxonClientException(
            HTTP_ERROR_MESSAGES_MAPPING.get(response.status_code)
        )

    def get(
        self,
        url,
        params=None,
        data=None,
        files=None,
        json_data=None,
        timeout=None,
        headers=None,
        session=None,
    ):
        """Call request with a get."""
        return self.request(
            "GET",
            url=url,
            params=params,
            data=data,
            files=files,
            json=json_data,
            timeout=timeout,
            headers=headers,
            session=session,
        )

    def post(
        self,
        url,
        params=None,
        data=None,
        files=None,
        json_data=None,
        timeout=None,
        headers=None,
        session=None,
    ):
        """Call request with a post."""
        return self.request(
            "POST",
            url=url,
            params=params,
            data=data,
            files=files,
            json=json_data,
            timeout=timeout,
            headers=headers,
            session=session,
        )

    def patch(
        self,
        url,
        params=None,
        data=None,
        files=None,
        json_data=None,
        timeout=None,
        headers=None,
        session=None,
    ):
        """Call request with a patch."""
        return self.request(
            "PATCH",
            url=url,
            params=params,
            data=data,
            files=files,
            json=json_data,
            timeout=timeout,
            headers=headers,
            session=session,
        )

    def delete(
        self,
        url,
        params=None,
        data=None,
        files=None,
        json_data=None,
        timeout=None,
        headers=None,
        session=None,
    ):
        """Call request with a delete."""
        return self.request(
            "DELETE",
            url=url,
            params=params,
            data=data,
            files=files,
            json=json_data,
            timeout=timeout,
            headers=headers,
            session=session,
        )

    def put(
        self,
        url,
        params=None,
        data=None,
        files=None,
        json_data=None,
        timeout=None,
        headers=None,
        session=None,
    ):
        """Call request with a put."""
        return self.request(
            "PUT",
            url=url,
            params=params,
            data=data,
            files=files,
            json=json_data,
            timeout=timeout,
            headers=headers,
            session=session,
        )



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from marshmallow import ValidationError, fields, validates_schema

from polyaxon.schemas.base import BaseCamelSchema, BaseConfig


def validate_security_context(user, group):
    if any([user, group]) and not all([user, group]):
        raise ValidationError(
            "Security context requires both `user` and `group` or none."
        )


class SecurityContextSchema(BaseCamelSchema):
    enabled = fields.Bool(allow_none=True)
    user = fields.Int(allow_none=True)
    group = fields.Int(allow_none=True)

    @staticmethod
    def schema_config():
        return SecurityContextConfig

    @validates_schema
    def validate_security_context(self, data, **kwargs):
        validate_security_context(data.get("user"), data.get("group"))


class SecurityContextConfig(BaseConfig):
    SCHEMA = SecurityContextSchema
    REDUCED_ATTRIBUTES = ["enabled", "user", "group"]

    def __init__(self, enabled=None, user=None, group=None):
        validate_security_context(user, group)
        self.enabled = enabled
        self.user = user
        self.group = group



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from polyaxon.utils.units import to_percentage
from tests.utils import BaseTestCase


class ToPercentageTest(BaseTestCase):
    """A test case for the `to_percentage`."""

    def test_works_as_expected_for_valid_values(self):
        test_data = [
            (0, "0%"),
            (0.25, "25%"),
            (-0.25, "-25%"),
            (12, "1200%"),
            (0.123, "12.3%"),
            (0.12345, "12.35%"),
            (0.12001, "12%"),
            (0.12101, "12.1%"),
            ("0", "0%"),
            ("0.25", "25%"),
        ]
        for value, expected in test_data:
            result = to_percentage(value)
            self.assertEqual(result, expected)

    def test_raises_value_error_for_invalid_types(self):
        with self.assertRaises(ValueError):
            to_percentage("foo")
